@@ -1,14 +1,14 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
-from __future__ import print_function
-from polyglot.builtins import map
+from polyglot.builtins import map, unicode_type, environ_item, hasenv, getenv, as_unicode, native_string_type
 import sys, locale, codecs, os, importlib, collections
 
-__appname__   = u'calibre'
-numeric_version = (3, 38, 1)
-__version__   = u'.'.join(map(unicode, numeric_version))
-__author__    = u"Kovid Goyal <kovid@kovidgoyal.net>"
+__appname__   = 'calibre'
+numeric_version = (5, 0, 1)
+__version__   = '.'.join(map(unicode_type, numeric_version))
+git_version   = None
+__author__    = "Kovid Goyal <kovid@kovidgoyal.net>"
 
 '''
 Various run time constants.
@@ -17,17 +17,17 @@ Various run time constants.
 
 _plat = sys.platform.lower()
 iswindows = 'win32' in _plat or 'win64' in _plat
-isosx     = 'darwin' in _plat
-isnewosx  = isosx and getattr(sys, 'new_app_bundle', False)
+ismacos = isosx = 'darwin' in _plat
+isnewosx  = ismacos and getattr(sys, 'new_app_bundle', False)
 isfreebsd = 'freebsd' in _plat
 isnetbsd = 'netbsd' in _plat
 isdragonflybsd = 'dragonfly' in _plat
 isbsd = isfreebsd or isnetbsd or isdragonflybsd
 ishaiku = 'haiku1' in _plat
-islinux   = not(iswindows or isosx or isbsd or ishaiku)
+islinux   = not(iswindows or ismacos or isbsd or ishaiku)
 isfrozen  = hasattr(sys, 'frozen')
-isunix = isosx or islinux or ishaiku
-isportable = os.environ.get('CALIBRE_PORTABLE_BUILD', None) is not None
+isunix = ismacos or islinux or ishaiku
+isportable = hasenv('CALIBRE_PORTABLE_BUILD')
 ispy3 = sys.version_info.major > 2
 isxp = isoldvista = False
 if iswindows:
@@ -35,13 +35,15 @@ if iswindows:
     isxp = wver.major < 6
     isoldvista = wver.build < 6002
 is64bit = sys.maxsize > (1 << 32)
-isworker = 'CALIBRE_WORKER' in os.environ or 'CALIBRE_SIMPLE_WORKER' in os.environ
+isworker = hasenv('CALIBRE_WORKER') or hasenv('CALIBRE_SIMPLE_WORKER')
 if isworker:
-    os.environ.pop('CALIBRE_FORCE_ANSI', None)
-FAKE_PROTOCOL, FAKE_HOST = 'https', 'calibre-internal.invalid'
-VIEWER_APP_UID = u'com.calibre-ebook.viewer'
-EDITOR_APP_UID = u'com.calibre-ebook.edit-book'
-MAIN_APP_UID = u'com.calibre-ebook.main-gui'
+    os.environ.pop(environ_item('CALIBRE_FORCE_ANSI'), None)
+FAKE_PROTOCOL, FAKE_HOST = 'clbr', 'internal.invalid'
+VIEWER_APP_UID = 'com.calibre-ebook.viewer'
+EDITOR_APP_UID = 'com.calibre-ebook.edit-book'
+MAIN_APP_UID = 'com.calibre-ebook.main-gui'
+STORE_DIALOG_APP_UID = 'com.calibre-ebook.store-dialog'
+TOC_DIALOG_APP_UID = 'com.calibre-ebook.toc-editor'
 try:
     preferred_encoding = locale.getpreferredencoding()
     codecs.lookup(preferred_encoding)
@@ -52,6 +54,26 @@ win32event = importlib.import_module('win32event') if iswindows else None
 winerror   = importlib.import_module('winerror') if iswindows else None
 win32api   = importlib.import_module('win32api') if iswindows else None
 fcntl      = None if iswindows else importlib.import_module('fcntl')
+dark_link_color = '#6cb4ee'
+builtin_colors_light = {
+    'yellow': '#ffeb6b',
+    'green': '#c0ed72',
+    'blue': '#add8ff',
+    'red': '#ffb0ca',
+    'purple': '#d9b2ff',
+}
+builtin_colors_dark = {
+    'yellow': '#c18d18',
+    'green': '#306f50',
+    'blue': '#265589',
+    'red': '#a23e5a',
+    'purple': '#505088',
+}
+builtin_decorations = {
+    'wavy': {'text-decoration-style': 'wavy', 'text-decoration-color': 'red', 'text-decoration-line': 'underline'},
+    'strikeout': {'text-decoration-line': 'line-through', 'text-decoration-color': 'red'},
+}
+
 
 _osx_ver = None
 
@@ -66,8 +88,8 @@ def get_osx_version():
             ver = platform.mac_ver()[0].split('.')
             if len(ver) == 2:
                 ver.append(0)
-            _osx_ver = OSX(*(map(int, ver)))
-        except:
+            _osx_ver = OSX(*map(int, ver))  # no2to3
+        except Exception:
             _osx_ver = OSX(0, 0, 0)
     return _osx_ver
 
@@ -83,11 +105,11 @@ else:
             # bytestring if sys.getfilesystemencoding() == 'ascii', which is
             # just plain dumb. This is fixed by the icu.py module which, when
             # imported changes ascii to utf-8
-    except:
+    except Exception:
         filesystem_encoding = 'utf-8'
 
 
-DEBUG = b'CALIBRE_DEBUG' in os.environ
+DEBUG = hasenv('CALIBRE_DEBUG')
 
 
 def debug():
@@ -97,7 +119,7 @@ def debug():
 
 def _get_cache_dir():
     import errno
-    confcache = os.path.join(config_dir, u'caches')
+    confcache = os.path.join(config_dir, 'caches')
     try:
         os.makedirs(confcache)
     except EnvironmentError as err:
@@ -105,13 +127,9 @@ def _get_cache_dir():
             raise
     if isportable:
         return confcache
-    if 'CALIBRE_CACHE_DIRECTORY' in os.environ:
-        if iswindows:
-            ans = get_unicode_windows_env_var(u'CALIBRE_CACHE_DIRECTORY')
-        else:
-            ans = os.path.abspath(os.environ['CALIBRE_CACHE_DIRECTORY'])
-            if isinstance(ans, bytes):
-                ans = ans.decode(filesystem_encoding)
+    ccd = getenv('CALIBRE_CACHE_DIRECTORY')
+    if ccd is not None:
+        ans = os.path.abspath(ccd)
         try:
             os.makedirs(ans)
             return ans
@@ -122,13 +140,13 @@ def _get_cache_dir():
     if iswindows:
         w = plugins['winutil'][0]
         try:
-            candidate = os.path.join(w.special_folder_path(w.CSIDL_LOCAL_APPDATA), u'%s-cache'%__appname__)
+            candidate = os.path.join(w.special_folder_path(w.CSIDL_LOCAL_APPDATA), '%s-cache'%__appname__)
         except ValueError:
             return confcache
-    elif isosx:
-        candidate = os.path.join(os.path.expanduser(u'~/Library/Caches'), __appname__)
+    elif ismacos:
+        candidate = os.path.join(os.path.expanduser('~/Library/Caches'), __appname__)
     else:
-        candidate = os.environ.get('XDG_CACHE_HOME', u'~/.cache')
+        candidate = getenv('XDG_CACHE_HOME', '~/.cache')
         candidate = os.path.join(os.path.expanduser(candidate),
                                     __appname__)
         if isinstance(candidate, bytes):
@@ -147,9 +165,13 @@ def _get_cache_dir():
 def cache_dir():
     ans = getattr(cache_dir, 'ans', None)
     if ans is None:
-        ans = cache_dir.ans = _get_cache_dir()
+        ans = cache_dir.ans = os.path.realpath(_get_cache_dir())
     return ans
 
+
+plugins_loc = sys.extensions_location
+
+
 # plugins {{{
 
 
@@ -164,32 +186,27 @@ class Plugins(collections.Mapping):
                 'podofo',
                 'cPalmdoc',
                 'progress_indicator',
-                'chmlib',
                 'icu',
                 'speedup',
+                'html_as_json',
                 'unicode_names',
-                'zlib2',
-                'html',
+                'html_syntax_highlighter',
+                'hyphen',
                 'freetype',
                 'imageops',
-                'qt_hack',
                 'hunspell',
                 '_patiencediff_c',
                 'bzzdec',
                 'matcher',
                 'tokenizer',
                 'certgen',
-                'lzma_binding',
             ]
-        if not ispy3:
-            plugins.extend([
-                'monotonic',
-            ])
         if iswindows:
             plugins.extend(['winutil', 'wpd', 'winfonts'])
-        if isosx:
+        if ismacos:
             plugins.append('usbobserver')
-        if isfreebsd or ishaiku or islinux or isosx:
+            plugins.append('cocoa')
+        if isfreebsd or ishaiku or islinux or ismacos:
             plugins.append('libusb')
             plugins.append('libmtp')
         self.plugins = frozenset(plugins)
@@ -197,18 +214,22 @@ class Plugins(collections.Mapping):
     def load_plugin(self, name):
         if name in self._plugins:
             return
-        sys.path.insert(0, sys.extensions_location)
+        sys.path.insert(0, plugins_loc)
         try:
             del sys.modules[name]
         except KeyError:
             pass
+        plugin_err = ''
         try:
-            p, err = importlib.import_module(name), ''
+            p = importlib.import_module(name)
         except Exception as err:
             p = None
-            err = str(err)
-        self._plugins[name] = (p, err)
-        sys.path.remove(sys.extensions_location)
+            try:
+                plugin_err = unicode_type(err)
+            except Exception:
+                plugin_err = as_unicode(native_string_type(err), encoding=preferred_encoding, errors='replace')
+        self._plugins[name] = p, plugin_err
+        sys.path.remove(plugins_loc)
 
     def __iter__(self):
         return iter(self.plugins)
@@ -235,8 +256,9 @@ if plugins is None:
 
 CONFIG_DIR_MODE = 0o700
 
-if 'CALIBRE_CONFIG_DIRECTORY' in os.environ:
-    config_dir = os.path.abspath(os.environ['CALIBRE_CONFIG_DIRECTORY'])
+cconfd = getenv('CALIBRE_CONFIG_DIRECTORY')
+if cconfd is not None:
+    config_dir = os.path.abspath(cconfd)
 elif iswindows:
     if plugins['winutil'][0] is None:
         raise Exception(plugins['winutil'][1])
@@ -247,10 +269,10 @@ elif iswindows:
     if not config_dir or not os.access(config_dir, os.W_OK|os.X_OK):
         config_dir = os.path.expanduser('~')
     config_dir = os.path.join(config_dir, 'calibre')
-elif isosx:
+elif ismacos:
     config_dir = os.path.expanduser('~/Library/Preferences/calibre')
 else:
-    bdir = os.path.abspath(os.path.expanduser(os.environ.get('XDG_CONFIG_HOME', '~/.config')))
+    bdir = os.path.abspath(os.path.expanduser(getenv('XDG_CONFIG_HOME', '~/.config')))
     config_dir = os.path.join(bdir, 'calibre')
     try:
         os.makedirs(config_dir, mode=CONFIG_DIR_MODE)
@@ -273,16 +295,20 @@ else:
 # }}}
 
 
-dv = os.environ.get('CALIBRE_DEVELOP_FROM')
+dv = getenv('CALIBRE_DEVELOP_FROM')
 is_running_from_develop = bool(getattr(sys, 'frozen', False) and dv and os.path.abspath(dv) in sys.path)
 del dv
+in_develop_mode = getenv('CALIBRE_ENABLE_DEVELOP_MODE') == '1'
 
 
 def get_version():
     '''Return version string for display to user '''
-    v = __version__
-    if numeric_version[-1] == 0:
-        v = v[:-2]
+    if git_version is not None:
+        v = git_version
+    else:
+        v = __version__
+        if numeric_version[-1] == 0:
+            v = v[:-2]
     if is_running_from_develop:
         v += '*'
     if iswindows and is64bit:
@@ -294,12 +320,7 @@ def get_version():
 def get_portable_base():
     'Return path to the directory that contains calibre-portable.exe or None'
     if isportable:
-        return os.path.dirname(os.path.dirname(get_unicode_windows_env_var(u'CALIBRE_PORTABLE_BUILD')))
-
-
-def get_unicode_windows_env_var(name):
-    getenv = plugins['winutil'][0].getenv
-    return getenv(unicode(name))
+        return os.path.dirname(os.path.dirname(getenv('CALIBRE_PORTABLE_BUILD')))
 
 
 def get_windows_username():
