@@ -1,3 +1,5 @@
+
+
 # Copyright (c) 2007 Mike Higgins (Falstaff)
 # Modifications from the original:
 #    Copyright (C) 2007 Kovid Goyal <kovid@kovidgoyal.net>
@@ -36,34 +38,33 @@
 #                           Plot, Image (outside of ImageBlock),
 #                           EmpLine, EmpDots
 
-import os, re, codecs, operator
+import os, re, codecs, operator, io
 from xml.sax.saxutils import escape
 from datetime import date
-try:
-    from elementtree.ElementTree import (Element, SubElement)
-    Element, SubElement
-except ImportError:
-    from xml.etree.ElementTree import (Element, SubElement)
+from xml.etree.ElementTree import Element, SubElement, ElementTree
 
-from elements import ElementWriter
-from pylrf import (LrfWriter, LrfObject, LrfTag, LrfToc,
+from .pylrf import (LrfWriter, LrfObject, LrfTag, LrfToc,
         STREAM_COMPRESSED, LrfTagStream, LrfStreamBase, IMAGE_TYPE_ENCODING,
         BINDING_DIRECTION_ENCODING, LINE_TYPE_ENCODING, LrfFileStream,
         STREAM_FORCE_COMPRESSED)
 from calibre.utils.date import isoformat
 
-DEFAULT_SOURCE_ENCODING = "cp1252"      # defualt is us-windows character set
+DEFAULT_SOURCE_ENCODING = "cp1252"      # default is us-windows character set
 DEFAULT_GENREADING      = "fs"          # default is yes to both lrf and lrs
 
 from calibre import __appname__, __version__
 from calibre import entity_to_unicode
+from polyglot.builtins import string_or_bytes, unicode_type, iteritems, native_string_type
+
 
 class LrsError(Exception):
     pass
 
+
 class ContentError(Exception):
     pass
 
+
 def _checkExists(filename):
     if not os.path.exists(filename):
         raise LrsError("file '%s' not found" % filename)
@@ -92,7 +93,7 @@ def ElementWithReading(tag, text, reading=False):
 
     if text is None:
         readingText = ""
-    elif isinstance(text, basestring):
+    elif isinstance(text, string_or_bytes):
         readingText = text
     else:
         # assumed to be a sequence of (name, sortas)
@@ -108,11 +109,10 @@ def appendTextElements(e, contentsList, se):
     """ A helper function to convert text streams into the proper elements. """
 
     def uconcat(text, newText, se):
-        if type(newText) != type(text):
-            if type(text) is str:
-                text = text.decode(se)
-            else:
-                newText = newText.decode(se)
+        if isinstance(text, bytes):
+            text = text.decode(se)
+        if isinstance(newText, bytes):
+            newText = newText.decode(se)
 
         return text + newText
 
@@ -136,6 +136,7 @@ def appendTextElements(e, contentsList, se):
 
 class Delegator(object):
     """ A mixin class to create delegated methods that create elements. """
+
     def __init__(self, delegates):
         self.delegates = delegates
         self.delegatedMethods = []
@@ -150,7 +151,7 @@ class Delegator(object):
 
             """
             for setting in d.getSettings():
-                if isinstance(setting, basestring):
+                if isinstance(setting, string_or_bytes):
                     setting = (d, setting)
                 delegates = \
                         self.delegatedSettingsDict.setdefault(setting[1], [])
@@ -218,6 +219,7 @@ class Delegator(object):
 
 class LrsAttributes(object):
     """ A mixin class to handle default and user supplied attributes. """
+
     def __init__(self, defaults, alsoAllow=None, **settings):
         if alsoAllow is None:
             alsoAllow = []
@@ -226,8 +228,8 @@ class LrsAttributes(object):
             if name not in self.attrs and name not in alsoAllow:
                 raise LrsError("%s does not support setting %s" %
                         (self.__class__.__name__, name))
-            if type(value) is int:
-                value = str(value)
+            if isinstance(value, int):
+                value = unicode_type(value)
             self.attrs[name] = value
 
 
@@ -235,6 +237,7 @@ class LrsContainer(object):
     """ This class is a mixin class for elements that are contained in or
         contain an unknown number of other elements.
     """
+
     def __init__(self, validChildren):
         self.parent = None
         self.contents = []
@@ -286,7 +289,7 @@ class LrsContainer(object):
                     (content.__class__.__name__,
                     self.__class__.__name__))
 
-        if convertText and isinstance(content, basestring):
+        if convertText and isinstance(content, string_or_bytes):
             content = Text(content)
 
         content.setParent(self)
@@ -330,13 +333,13 @@ class LrsObject(object):
     def lrsObjectElement(self, name, objlabel="objlabel", labelName=None,
             labelDecorate=True, **settings):
         element = Element(name)
-        element.attrib["objid"] = str(self.objId)
+        element.attrib["objid"] = unicode_type(self.objId)
         if labelName is None:
             labelName = name
         if labelDecorate:
             label = "%s.%d" % (labelName, self.objId)
         else:
-            label = str(self.objId)
+            label = unicode_type(self.objId)
         element.attrib[objlabel] = label
         element.attrib.update(settings)
         return element
@@ -418,7 +421,7 @@ class Book(Delegator):
         LrsObject.nextObjId += 1
 
         styledefault = StyleDefault()
-        if settings.has_key('setdefault'):  # noqa
+        if 'setdefault' in settings:
             styledefault = settings.pop('setdefault')
         Delegator.__init__(self, [BookInformation(), Main(),
             Template(), Style(styledefault), Solos(), Objects()])
@@ -555,22 +558,23 @@ class Book(Delegator):
             length = len(text.text)
             fonts[fs] = fonts.get(fs, 0) + length
         if not fonts:
-            print 'WARNING: LRF seems to have no textual content. Cannot rationalize font sizes.'
+            print('WARNING: LRF seems to have no textual content. Cannot rationalize font sizes.')
             return
 
         old_base_font_size = float(max(fonts.items(), key=operator.itemgetter(1))[0])
         factor = base_font_size / old_base_font_size
+
         def rescale(old):
-            return str(int(int(old) * factor))
+            return unicode_type(int(int(old) * factor))
 
         text_blocks = list(main.get_all(lambda x: isinstance(x, TextBlock)))
         for tb in text_blocks:
-            if tb.textSettings.has_key('fontsize'):  # noqa
+            if 'fontsize' in tb.textSettings:
                 tb.textSettings['fontsize'] = rescale(tb.textSettings['fontsize'])
             for span in tb.get_all(lambda x: isinstance(x, Span)):
-                if span.attrs.has_key('fontsize'):  # noqa
+                if 'fontsize' in span.attrs:
                     span.attrs['fontsize'] = rescale(span.attrs['fontsize'])
-                if span.attrs.has_key('baselineskip'):  # noqa
+                if 'baselineskip' in span.attrs:
                     span.attrs['baselineskip'] = rescale(span.attrs['baselineskip'])
 
         text_styles = set(tb.textStyle for tb in text_blocks)
@@ -579,15 +583,15 @@ class Book(Delegator):
             ts.attrs['baselineskip'] = rescale(ts.attrs['baselineskip'])
 
     def renderLrs(self, lrsFile, encoding="UTF-8"):
-        if isinstance(lrsFile, basestring):
+        if isinstance(lrsFile, string_or_bytes):
             lrsFile = codecs.open(lrsFile, "wb", encoding=encoding)
         self.render(lrsFile, outputEncodingName=encoding)
         lrsFile.close()
 
     def renderLrf(self, lrfFile):
         self.appendReferencedObjects(self)
-        if isinstance(lrfFile, basestring):
-            lrfFile = file(lrfFile, "wb")
+        if isinstance(lrfFile, string_or_bytes):
+            lrfFile = open(lrfFile, "wb")
         lrfWriter = LrfWriter(self.sourceencoding)
 
         lrfWriter.optimizeTags = self.optimizeTags
@@ -615,16 +619,13 @@ class Book(Delegator):
         # now, add some newlines to make it easier to look at
 
         _formatXml(root)
-
-        writer = ElementWriter(root, header=True,
-                               sourceEncoding=self.sourceencoding,
-                               spaceBeforeClose=False,
-                               outputEncodingName=outputEncodingName)
-        writer.write(f)
+        tree = ElementTree(element=root)
+        tree.write(f, encoding=native_string_type(outputEncodingName), xml_declaration=True)
 
 
 class BookInformation(Delegator):
     """ Just a container for the Info and TableOfContents elements. """
+
     def __init__(self):
         Delegator.__init__(self, [Info(), TableOfContents()])
 
@@ -636,6 +637,7 @@ class BookInformation(Delegator):
 
 class Info(Delegator):
     """ Just a container for the BookInfo and DocInfo elements. """
+
     def __init__(self):
         self.genreading = DEFAULT_GENREADING
         Delegator.__init__(self, [BookInfo(), DocInfo()])
@@ -668,9 +670,10 @@ class Info(Delegator):
 
         # fix up the doc info to match the LRF format
         # NB: generates an encoding attribute, which lrs2lrf does not
-        xmlInfo = ElementWriter(info, header=True, sourceEncoding=lrfWriter.getSourceEncoding(),
-                                spaceBeforeClose=False).toString()
-
+        tree = ElementTree(element=info)
+        f = io.BytesIO()
+        tree.write(f, encoding=native_string_type('utf-8'), xml_declaration=True)
+        xmlInfo = f.getvalue().decode('utf-8')
         xmlInfo = re.sub(r"<CThumbnail.*?>\n", "", xmlInfo)
         xmlInfo = xmlInfo.replace("SumPage>", "Page>")
         lrfWriter.docInfoXml = xmlInfo
@@ -693,7 +696,7 @@ class TableOfContents(object):
     def addTocEntry(self, tocLabel, textBlock):
         if not isinstance(textBlock, (Canvas, TextBlock, ImageBlock, RuledLine)):
             raise LrsError("TOC destination must be a Canvas, TextBlock, ImageBlock or RuledLine"+
-                            " not a " + str(type(textBlock)))
+                            " not a " + unicode_type(type(textBlock)))
 
         if textBlock.parent is None:
             raise LrsError("TOC text block must be already appended to a page")
@@ -743,8 +746,8 @@ class TocLabel(object):
 
     def toElement(self, se):
         return ElementWithText("TocLabel", self.label,
-                 refobj=str(self.textBlock.objId),
-                 refpage=str(self.textBlock.parent.objId))
+                 refobj=unicode_type(self.textBlock.objId),
+                 refpage=unicode_type(self.textBlock.parent.objId))
 
 
 class BookInfo(object):
@@ -805,7 +808,7 @@ class DocInfo(object):
         self.thumbnail = None
         self.language = "en"
         self.creator  = None
-        self.creationdate = str(isoformat(date.today()))
+        self.creationdate = unicode_type(isoformat(date.today()))
         self.producer = "%s v%s"%(__appname__, __version__)
         self.numberofpages = "0"
 
@@ -829,7 +832,7 @@ class DocInfo(object):
         docInfo.append(ElementWithText("Creator", self.creator))
         docInfo.append(ElementWithText("CreationDate", self.creationdate))
         docInfo.append(ElementWithText("Producer", self.producer))
-        docInfo.append(ElementWithText("SumPage", str(self.numberofpages)))
+        docInfo.append(ElementWithText("SumPage", unicode_type(self.numberofpages)))
         return docInfo
 
 
@@ -940,6 +943,7 @@ class Template(object):
         # does nothing
         pass
 
+
 class StyleDefault(LrsAttributes):
     """
         Supply some defaults for all TextBlocks.
@@ -1079,6 +1083,7 @@ class BookSetting(LrsAttributes):
 
 class LrsStyle(LrsObject, LrsAttributes, LrsContainer):
     """ A mixin class for styles. """
+
     def __init__(self, elementName, defaults=None, alsoAllow=None, **overrides):
         if defaults is None:
             defaults = {}
@@ -1089,7 +1094,7 @@ class LrsStyle(LrsObject, LrsAttributes, LrsContainer):
         self.elementName = elementName
         self.objectsAppended = False
         # self.label = "%s.%d" % (elementName, self.objId)
-        # self.label = str(self.objId)
+        # self.label = unicode_type(self.objId)
         # self.parent = None
 
     def update(self, settings):
@@ -1099,11 +1104,11 @@ class LrsStyle(LrsObject, LrsAttributes, LrsContainer):
             self.attrs[name] = value
 
     def getLabel(self):
-        return str(self.objId)
+        return unicode_type(self.objId)
 
     def toElement(self, se):
         element = Element(self.elementName, stylelabel=self.getLabel(),
-                objid=str(self.objId))
+                objid=unicode_type(self.objId))
         element.attrib.update(self.attrs)
         return element
 
@@ -1117,6 +1122,7 @@ class LrsStyle(LrsObject, LrsAttributes, LrsContainer):
             return self.__class__ == other.__class__ and self.attrs == other.attrs
         return False
 
+
 class TextStyle(LrsStyle):
     """
         The text style of a TextBlock.  Default is 10 pt. Times Roman.
@@ -1147,7 +1153,7 @@ class TextStyle(LrsStyle):
                  "rubyadjust", "rubyalign", "rubyoverhang",
                  "empdotsposition", 'emplinetype', 'emplineposition']
 
-    validSettings = baseDefaults.keys() + alsoAllow
+    validSettings = list(baseDefaults) + alsoAllow
 
     defaults = baseDefaults.copy()
 
@@ -1208,7 +1214,7 @@ class PageStyle(LrsStyle):
     alsoAllow = ["header", "evenheader", "oddheader",
                  "footer", "evenfooter", "oddfooter"]
 
-    validSettings = baseDefaults.keys() + alsoAllow
+    validSettings = list(baseDefaults) + alsoAllow
     defaults = baseDefaults.copy()
 
     @classmethod
@@ -1230,14 +1236,14 @@ class PageStyle(LrsStyle):
             del settings[evenbase]
             if evenObj.parent is None:
                 parent.append(evenObj)
-            settings[evenbase + "id"] = str(evenObj.objId)
+            settings[evenbase + "id"] = unicode_type(evenObj.objId)
 
         if oddbase in settings:
             oddObj = settings[oddbase]
             del settings[oddbase]
             if oddObj.parent is None:
                 parent.append(oddObj)
-            settings[oddbase + "id"] = str(oddObj.objId)
+            settings[oddbase + "id"] = unicode_type(oddObj.objId)
 
     def appendReferencedObjects(self, parent):
         if self.objectsAppended:
@@ -1477,11 +1483,12 @@ class Paragraph(LrsContainer):
         the things that can go in it.)  It's less confusing (to me) to use
         explicit .append methods to build up the text stream.
     """
+
     def __init__(self, text=None):
         LrsContainer.__init__(self, [Text, CR, DropCaps, CharButton,
-                                     LrsSimpleChar1, basestring])
+                                     LrsSimpleChar1, bytes, unicode_type])
         if text is not None:
-            if isinstance(text, basestring):
+            if isinstance(text, string_or_bytes):
                 text = Text(text)
             self.append(text)
 
@@ -1514,7 +1521,7 @@ class Paragraph(LrsContainer):
 class LrsTextTag(LrsContainer):
 
     def __init__(self, text, validContents):
-        LrsContainer.__init__(self, [Text, basestring] + validContents)
+        LrsContainer.__init__(self, [Text, bytes, unicode_type] + validContents)
         if text is not None:
             self.append(text)
 
@@ -1573,7 +1580,7 @@ class DropCaps(LrsTextTag):
         return self.text is None or not self.text.strip()
 
     def toElement(self, se):
-        elem =  Element('DrawChar', line=str(self.line))
+        elem =  Element('DrawChar', line=unicode_type(self.line))
         appendTextElements(elem, self.contents, se)
         return elem
 
@@ -1620,9 +1627,11 @@ class Button(LrsObject, LrsContainer):
 
         return b
 
+
 class ButtonBlock(Button):
     pass
 
+
 class PushButton(LrsContainer):
 
     def __init__(self, **settings):
@@ -1636,6 +1645,7 @@ class PushButton(LrsContainer):
 
         return b
 
+
 class JumpTo(LrsContainer):
 
     def __init__(self, textBlock):
@@ -1646,7 +1656,7 @@ class JumpTo(LrsContainer):
         self.textBlock = textBlock
 
     def toElement(self, se):
-        return Element("JumpTo", refpage=str(self.textBlock.parent.objId), refobj=str(self.textBlock.objId))
+        return Element("JumpTo", refpage=unicode_type(self.textBlock.parent.objId), refobj=unicode_type(self.textBlock.objId))
 
 
 class Plot(LrsSimpleChar1, LrsContainer):
@@ -1678,8 +1688,8 @@ class Plot(LrsSimpleChar1, LrsContainer):
             parent.append(self.obj)
 
     def toElement(self, se):
-        elem =  Element('Plot', xsize=str(self.xsize), ysize=str(self.ysize),
-                                refobj=str(self.obj.objId))
+        elem =  Element('Plot', xsize=unicode_type(self.xsize), ysize=unicode_type(self.ysize),
+                                refobj=unicode_type(self.obj.objId))
         if self.adjustment:
             elem.set('adjustment', self.adjustment)
         return elem
@@ -1690,8 +1700,10 @@ class Plot(LrsSimpleChar1, LrsContainer):
                   Plot.ADJUSTMENT_VALUES[adj])
         parent.appendLrfTag(LrfTag("Plot", params))
 
+
 class Text(LrsContainer):
     """ A object that represents raw text.  Does not have a toElement. """
+
     def __init__(self, text):
         LrsContainer.__init__(self, [])
         self.text = text
@@ -1701,7 +1713,7 @@ class Text(LrsContainer):
 
     def toLrfContainer(self, lrfWriter, parent):
         if self.text:
-            if isinstance(self.text, str):
+            if isinstance(self.text, bytes):
                 parent.appendLrfTag(LrfTag("rawtext", self.text))
             else:
                 parent.appendLrfTag(LrfTag("textstring", self.text))
@@ -1712,6 +1724,7 @@ class CR(LrsSimpleChar1, LrsContainer):
         A line break (when appended to a Paragraph) or a paragraph break
         (when appended to a TextBlock).
     """
+
     def __init__(self):
         LrsContainer.__init__(self, [])
 
@@ -1727,6 +1740,7 @@ class Italic(LrsSimpleChar1, LrsTextTag):
     def __init__(self, text=None):
         LrsTextTag.__init__(self, text, [LrsSimpleChar1])
 
+
 class Sub(LrsSimpleChar1, LrsTextTag):
 
     def __init__(self, text=None):
@@ -1757,7 +1771,7 @@ class Space(LrsSimpleChar1, LrsContainer):
         if self.xsize == 0:
             return
 
-        return Element("Space", xsize=str(self.xsize))
+        return Element("Space", xsize=unicode_type(self.xsize))
 
     def toLrfContainer(self, lrfWriter, container):
         if self.xsize != 0:
@@ -1769,8 +1783,9 @@ class Box(LrsSimpleChar1, LrsContainer):
         Draw a box around text.  Unfortunately, does not seem to do
         anything on the PRS-500.
     """
+
     def __init__(self, linetype="solid"):
-        LrsContainer.__init__(self, [Text, basestring])
+        LrsContainer.__init__(self, [Text, bytes, unicode_type])
         if linetype not in LINE_TYPE_ENCODING:
             raise LrsError(linetype + " is not a valid line type")
         self.linetype = linetype
@@ -1790,9 +1805,9 @@ class Box(LrsSimpleChar1, LrsContainer):
 class Span(LrsSimpleChar1, LrsContainer):
 
     def __init__(self, text=None, **attrs):
-        LrsContainer.__init__(self, [LrsSimpleChar1, Text, basestring])
+        LrsContainer.__init__(self, [LrsSimpleChar1, Text, bytes, unicode_type])
         if text is not None:
-            if isinstance(text, basestring):
+            if isinstance(text, string_or_bytes):
                 text = Text(text)
             self.append(text)
 
@@ -1820,7 +1835,7 @@ class Span(LrsSimpleChar1, LrsContainer):
         oldTextStyle = self.findCurrentTextStyle()
 
         # set the attributes we want changed
-        for (name, value) in self.attrs.items():
+        for (name, value) in tuple(iteritems(self.attrs)):
             if name in oldTextStyle.attrs and oldTextStyle.attrs[name] == self.attrs[name]:
                 self.attrs.pop(name)
             else:
@@ -1843,11 +1858,12 @@ class Span(LrsSimpleChar1, LrsContainer):
     def toElement(self, se):
         element = Element('Span')
         for (key, value) in self.attrs.items():
-            element.set(key, str(value))
+            element.set(key, unicode_type(value))
 
         appendTextElements(element, self.contents, se)
         return element
 
+
 class EmpLine(LrsTextTag, LrsSimpleChar1):
     emplinetypes = ['none', 'solid', 'dotted', 'dashed', 'double']
     emplinepositions = ['before', 'after']
@@ -1855,9 +1871,9 @@ class EmpLine(LrsTextTag, LrsSimpleChar1):
     def __init__(self, text=None, emplineposition='before', emplinetype='solid'):
         LrsTextTag.__init__(self, text, [LrsSimpleChar1])
         if emplineposition not in self.__class__.emplinepositions:
-            raise LrsError('emplineposition for an EmpLine must be one of: '+str(self.__class__.emplinepositions))
+            raise LrsError('emplineposition for an EmpLine must be one of: '+unicode_type(self.__class__.emplinepositions))
         if emplinetype not in self.__class__.emplinetypes:
-            raise LrsError('emplinetype for an EmpLine must be one of: '+str(self.__class__.emplinetypes))
+            raise LrsError('emplinetype for an EmpLine must be one of: '+unicode_type(self.__class__.emplinetypes))
 
         self.emplinetype     = emplinetype
         self.emplineposition = emplineposition
@@ -1879,11 +1895,13 @@ class EmpLine(LrsTextTag, LrsSimpleChar1):
         appendTextElements(element, self.contents, se)
         return element
 
+
 class Bold(Span):
     """
         There is no known "bold" lrf tag. Use Span with a fontweight in LRF,
         but use the word Bold in the LRS.
     """
+
     def __init__(self, text=None):
         Span.__init__(self, text, fontweight=800)
 
@@ -1895,6 +1913,7 @@ class Bold(Span):
 
 class BlockSpace(LrsContainer):
     """ Can be appended to a page to move the text point. """
+
     def __init__(self, xspace=0, yspace=0, x=0, y=0):
         LrsContainer.__init__(self, [])
         if xspace == 0 and x != 0:
@@ -1914,9 +1933,9 @@ class BlockSpace(LrsContainer):
         element = Element("BlockSpace")
 
         if self.xspace != 0:
-            element.attrib["xspace"] = str(self.xspace)
+            element.attrib["xspace"] = unicode_type(self.xspace)
         if self.yspace != 0:
-            element.attrib["yspace"] = str(self.yspace)
+            element.attrib["yspace"] = unicode_type(self.yspace)
 
         return element
 
@@ -1928,8 +1947,9 @@ class CharButton(LrsSimpleChar1, LrsContainer):
 
         Only text or SimpleChars can be appended to the CharButton.
     """
+
     def __init__(self, button, text=None):
-        LrsContainer.__init__(self, [basestring, Text, LrsSimpleChar1])
+        LrsContainer.__init__(self, [bytes, unicode_type, Text, LrsSimpleChar1])
         self.button = None
         if button is not None:
             self.setButton(button)
@@ -1959,7 +1979,7 @@ class CharButton(LrsSimpleChar1, LrsContainer):
         container.appendLrfTag(LrfTag("CharButtonEnd"))
 
     def toElement(self, se):
-        cb = Element("CharButton", refobj=str(self.button.objId))
+        cb = Element("CharButton", refobj=unicode_type(self.button.objId))
         appendTextElements(cb, self.contents, se)
         return cb
 
@@ -2037,6 +2057,7 @@ class JumpButton(LrsObject, LrsContainer):
         Actually creates several elements in the XML.  JumpButtons must
         be eventually appended to a Book (actually, an Object.)
     """
+
     def __init__(self, textBlock):
         LrsObject.__init__(self)
         LrsContainer.__init__(self, [])
@@ -2060,8 +2081,8 @@ class JumpButton(LrsObject, LrsContainer):
         b = self.lrsObjectElement("Button")
         pb = SubElement(b, "PushButton")
         SubElement(pb, "JumpTo",
-            refpage=str(self.textBlock.parent.objId),
-            refobj=str(self.textBlock.objId))
+            refpage=unicode_type(self.textBlock.parent.objId),
+            refobj=unicode_type(self.textBlock.objId))
         return b
 
 
@@ -2140,6 +2161,7 @@ class Header(HeaderOrFooter):
 class Footer(HeaderOrFooter):
     pass
 
+
 class Canvas(LrsObject, LrsContainer, LrsAttributes):
     defaults = dict(framemode="square", layout="LrTb", framewidth="0",
                 framecolor="0x00000000", bgcolor="0xFF000000",
@@ -2208,8 +2230,8 @@ class PutObj(LrsContainer):
             self.content.objId)))
 
     def toElement(self, se):
-        el = Element("PutObj", x1=str(self.x1), y1=str(self.y1),
-                    refobj=str(self.content.objId))
+        el = Element("PutObj", x1=unicode_type(self.x1), y1=unicode_type(self.y1),
+                    refobj=unicode_type(self.content.objId))
         return el
 
 
@@ -2246,9 +2268,8 @@ class ImageStream(LrsObject, LrsContainer):
         self.encoding = encoding
 
     def toLrf(self, lrfWriter):
-        imageFile = file(self.filename, "rb")
-        imageData = imageFile.read()
-        imageFile.close()
+        with open(self.filename, "rb") as f:
+            imageData = f.read()
 
         isObj = LrfObject("ImageStream", self.objId)
         if self.comment is not None:
@@ -2266,6 +2287,7 @@ class ImageStream(LrsObject, LrsContainer):
         element.text = self.comment
         return element
 
+
 class Image(LrsObject, LrsContainer, LrsAttributes):
 
     defaults = dict()
@@ -2291,9 +2313,9 @@ class Image(LrsObject, LrsContainer, LrsAttributes):
 
     def toElement(self, se):
         element = self.lrsObjectElement("Image", **self.attrs)
-        element.set("refstream", str(self.refstream.objId))
+        element.set("refstream", unicode_type(self.refstream.objId))
         for name in ["x0", "y0", "x1", "y1", "xsize", "ysize"]:
-            element.set(name, str(getattr(self, name)))
+            element.set(name, unicode_type(getattr(self, name)))
         return element
 
     def toLrf(self, lrfWriter):
@@ -2374,15 +2396,16 @@ class ImageBlock(LrsObject, LrsContainer, LrsAttributes):
 
     def toElement(self, se):
         element = self.lrsObjectElement("ImageBlock", **self.attrs)
-        element.set("refstream", str(self.refstream.objId))
+        element.set("refstream", unicode_type(self.refstream.objId))
         for name in ["x0", "y0", "x1", "y1", "xsize", "ysize"]:
-            element.set(name, str(getattr(self, name)))
+            element.set(name, unicode_type(getattr(self, name)))
         element.text = self.alttext
         return element
 
 
 class Font(LrsContainer):
     """ Allows a TrueType file to be embedded in an Lrf. """
+
     def __init__(self, file=None, fontname=None, fontfilename=None, encoding=None):
         LrsContainer.__init__(self, [])
         try:
