@@ -1,9 +1,12 @@
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
+
 """ The GUI """
+
 import glob
 import os
-import Queue
 import signal
 import sys
 import threading
@@ -12,18 +15,18 @@ from threading import Lock, RLock
 
 from PyQt5.Qt import (
     QT_VERSION, QApplication, QBuffer, QByteArray, QCoreApplication, QDateTime,
-    QDesktopServices, QDialog, QEvent, QFileDialog, QFileIconProvider, QFileInfo,
-    QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale, QObject,
-    QSettings, QSocketNotifier, QStringListModel, Qt, QThread, QTimer, QTranslator,
-    QUrl, pyqtSignal
+    QDesktopServices, QDialog, QEvent, QFileDialog, QFileIconProvider, QFileInfo, QPalette,
+    QFont, QFontDatabase, QFontInfo, QFontMetrics, QIcon, QLocale, QColor,
+    QNetworkProxyFactory, QObject, QSettings, QSocketNotifier, QStringListModel, Qt,
+    QThread, QTimer, QTranslator, QUrl, pyqtSignal
 )
 from PyQt5.QtWidgets import QStyle  # Gives a nicer error message than import from Qt
 
 from calibre import as_unicode, prints
 from calibre.constants import (
     DEBUG, __appname__ as APP_UID, __version__, config_dir, filesystem_encoding,
-    is_running_from_develop, isbsd, isfrozen, islinux, isosx, iswindows, isxp,
-    plugins
+    is_running_from_develop, isbsd, isfrozen, islinux, ismacos, iswindows, isxp,
+    plugins, plugins_loc
 )
 from calibre.ebooks.metadata import MetaInformation
 from calibre.gui2.linux_file_dialogs import (
@@ -35,6 +38,10 @@ from calibre.utils.config import Config, ConfigProxy, JSONConfig, dynamic
 from calibre.utils.date import UNDEFINED_DATE
 from calibre.utils.file_type_icons import EXT_MAP
 from calibre.utils.localization import get_lang
+from polyglot import queue
+from polyglot.builtins import (
+    iteritems, itervalues, range, string_or_bytes, unicode_type, map
+)
 
 try:
     NO_URL_FORMATTING = QUrl.None_
@@ -62,7 +69,7 @@ native_menubar_defaults = {
 
 def create_defs():
     defs = gprefs.defaults
-    if isosx:
+    if ismacos:
         defs['action-layout-menubar'] = native_menubar_defaults['action-layout-menubar']
         defs['action-layout-menubar-device'] = native_menubar_defaults['action-layout-menubar-device']
         defs['action-layout-toolbar'] = (
@@ -140,9 +147,10 @@ def create_defs():
     defs['blocked_auto_formats'] = []
     defs['auto_add_auto_convert'] = True
     defs['auto_add_everything'] = False
-    defs['ui_style'] = 'calibre' if iswindows or isosx else 'system'
+    defs['ui_style'] = 'calibre' if iswindows or ismacos else 'system'
     defs['tag_browser_old_look'] = False
     defs['tag_browser_hide_empty_categories'] = False
+    defs['tag_browser_always_autocollapse'] = False
     defs['book_list_tooltips'] = True
     defs['show_layout_buttons'] = False
     defs['bd_show_cover'] = True
@@ -179,11 +187,15 @@ def create_defs():
     defs['qv_dclick_changes_column'] = True
     defs['qv_retkey_changes_column'] = True
     defs['qv_follows_column'] = False
-    defs['book_details_narrow_comments_layout'] = 'float'
+    defs['book_details_comments_heading_pos'] = 'hide'
     defs['book_list_split'] = False
     defs['wrap_toolbar_text'] = False
     defs['dnd_merge'] = True
     defs['booklist_grid'] = False
+    defs['browse_annots_restrict_to_user'] = None
+    defs['browse_annots_restrict_to_type'] = None
+    defs['browse_annots_use_stemmer'] = True
+    defs['annots_export_format'] = 'txt'
 
 
 create_defs()
@@ -191,7 +203,7 @@ del create_defs
 # }}}
 
 UNDEFINED_QDATETIME = QDateTime(UNDEFINED_DATE)
-
+QT_HIDDEN_CLEAR_ACTION = '_q_qlineeditclearaction'
 ALL_COLUMNS = ['title', 'ondevice', 'authors', 'size', 'timestamp', 'rating', 'publisher',
         'tags', 'series', 'pubdate']
 
@@ -319,7 +331,7 @@ def default_author_link():
 
 def available_heights():
     desktop  = QCoreApplication.instance().desktop()
-    return map(lambda x: x.height(), map(desktop.availableGeometry, range(desktop.screenCount())))
+    return list(map(lambda x: x.height(), map(desktop.availableGeometry, range(desktop.screenCount()))))
 
 
 def available_height():
@@ -352,7 +364,7 @@ def is_widescreen():
     global _is_widescreen
     if _is_widescreen is None:
         try:
-            _is_widescreen = float(available_width())/available_height() > 1.4
+            _is_widescreen = available_width()/available_height() > 1.4
         except:
             _is_widescreen = False
     return _is_widescreen
@@ -385,21 +397,26 @@ def error_dialog(parent, title, msg, det_msg='', show=False,
 
 
 def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
-        default_yes=True,
-        # Skippable dialogs
-        # Set skip_dialog_name to a unique name for this dialog
-        # Set skip_dialog_msg to a message displayed to the user
-        skip_dialog_name=None, skip_dialog_msg=_('Show this confirmation again'),
-        skip_dialog_skipped_value=True, skip_dialog_skip_precheck=True,
-        # Override icon (QIcon to be used as the icon for this dialog or string for I())
-        override_icon=None,
-        # Change the text/icons of the yes and no buttons.
-        # The icons must be QIcon objects or strings for I()
-        yes_text=None, no_text=None, yes_icon=None, no_icon=None,
-    ):
+    default_yes=True,
+    # Skippable dialogs
+    # Set skip_dialog_name to a unique name for this dialog
+    # Set skip_dialog_msg to a message displayed to the user
+    skip_dialog_name=None, skip_dialog_msg=_('Show this confirmation again'),
+    skip_dialog_skipped_value=True, skip_dialog_skip_precheck=True,
+    # Override icon (QIcon to be used as the icon for this dialog or string for I())
+    override_icon=None,
+    # Change the text/icons of the yes and no buttons.
+    # The icons must be QIcon objects or strings for I()
+    yes_text=None, no_text=None, yes_icon=None, no_icon=None,
+):
     from calibre.gui2.dialogs.message_box import MessageBox
 
-    auto_skip = set(gprefs.get('questions_to_auto_skip', []))
+    if not isinstance(skip_dialog_name, unicode_type):
+        skip_dialog_name = None
+    try:
+        auto_skip = set(gprefs.get('questions_to_auto_skip', ()))
+    except Exception:
+        auto_skip = set()
     if (skip_dialog_name is not None and skip_dialog_name in auto_skip):
         return bool(skip_dialog_skipped_value)
 
@@ -501,7 +518,7 @@ class FunctionDispatcher(QObject):
         if not queued:
             typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
         self.dispatch_signal.connect(self.dispatch, type=typ)
-        self.q = Queue.Queue()
+        self.q = queue.Queue()
         self.lock = threading.Lock()
 
     def __call__(self, *args, **kwargs):
@@ -568,7 +585,7 @@ class FileIconProvider(QFileIconProvider):
         upath, bpath = I('mimetypes'), I('mimetypes', allow_user_override=False)
         if upath != bpath:
             # User has chosen to override mimetype icons
-            path_map = {v:I('mimetypes/%s.png' % v) for v in set(self.ICONS.itervalues())}
+            path_map = {v:I('mimetypes/%s.png' % v) for v in set(itervalues(self.ICONS))}
             icons = self.ICONS.copy()
             for uicon in glob.glob(os.path.join(upath, '*.png')):
                 ukey = os.path.basename(uicon).rpartition('.')[0].lower()
@@ -576,18 +593,18 @@ class FileIconProvider(QFileIconProvider):
                     path_map[ukey] = uicon
                     icons[ukey] = ukey
         else:
-            path_map = {v:os.path.join(bpath, v + '.png') for v in set(self.ICONS.itervalues())}
+            path_map = {v:os.path.join(bpath, v + '.png') for v in set(itervalues(self.ICONS))}
             icons = self.ICONS
-        self.icons = {k:path_map[v] for k, v in icons.iteritems()}
+        self.icons = {k:path_map[v] for k, v in iteritems(icons)}
         self.icons['calibre'] = I('lt.png', allow_user_override=False)
         for i in ('dir', 'default', 'zero'):
             self.icons[i] = QIcon(self.icons[i])
 
     def key_from_ext(self, ext):
-        key = ext if ext in self.icons.keys() else 'default'
+        key = ext if ext in list(self.icons.keys()) else 'default'
         if key == 'default' and ext.count('.') > 0:
             ext = ext.rpartition('.')[2]
-            key = ext if ext in self.icons.keys() else 'default'
+            key = ext if ext in list(self.icons.keys()) else 'default'
         return key
 
     def cached_icon(self, key):
@@ -612,7 +629,7 @@ class FileIconProvider(QFileIconProvider):
         if fileinfo.isDir():
             key = 'dir'
         else:
-            ext = unicode(fileinfo.completeSuffix()).lower()
+            ext = unicode_type(fileinfo.completeSuffix()).lower()
             key = self.key_from_ext(ext)
         return self.cached_icon(key)
 
@@ -646,7 +663,7 @@ if iswindows and 'CALIBRE_NO_NATIVE_FILEDIALOGS' not in os.environ:
     from calibre.gui2.win_file_dialogs import is_ok as has_windows_file_dialog_helper
     has_windows_file_dialog_helper = has_windows_file_dialog_helper()
 has_linux_file_dialog_helper = False
-if not iswindows and not isosx and 'CALIBRE_NO_NATIVE_FILEDIALOGS' not in os.environ and getattr(sys, 'frozen', False):
+if not iswindows and not ismacos and 'CALIBRE_NO_NATIVE_FILEDIALOGS' not in os.environ and getattr(sys, 'frozen', False):
     has_linux_file_dialog_helper = check_for_linux_native_dialogs()
 
 if has_windows_file_dialog_helper:
@@ -659,6 +676,12 @@ else:
     choose_files, choose_images, choose_dir, choose_save_file
 
 
+def is_dark_theme():
+    pal = QApplication.instance().palette()
+    col = pal.color(pal.Window)
+    return max(col.getRgb()[:3]) < 115
+
+
 def choose_osx_app(window, name, title, default_dir='/Applications'):
     fd = FileDialog(title=title, parent=window, name=name, mode=QFileDialog.ExistingFile,
             default_dir=default_dir)
@@ -685,7 +708,7 @@ def pixmap_to_data(pixmap, format='JPEG', quality=None):
     buf = QBuffer(ba)
     buf.open(QBuffer.WriteOnly)
     pixmap.save(buf, format, quality=quality)
-    return bytes(ba.data())
+    return ba.data()
 
 
 def decouple(prefix):
@@ -732,30 +755,30 @@ class Translator(QTranslator):
 
     def translate(self, *args, **kwargs):
         try:
-            src = unicode(args[1])
+            src = unicode_type(args[1])
         except:
-            return u''
+            return ''
         t = _
         return t(src)
 
 
 gui_thread = None
-
 qt_app = None
 
-builtin_fonts_loaded = False
+
+def calibre_font_files():
+    return glob.glob(P('fonts/liberation/*.?tf')) + [P('fonts/calibreSymbols.otf')] + \
+            glob.glob(os.path.join(config_dir, 'fonts', '*.?tf'))
 
 
 def load_builtin_fonts():
     global _rating_font, builtin_fonts_loaded
     # Load the builtin fonts and any fonts added to calibre by the user to
     # Qt
-    if builtin_fonts_loaded:
+    if hasattr(load_builtin_fonts, 'done'):
         return
-    builtin_fonts_loaded = True
-    for ff in glob.glob(P('fonts/liberation/*.?tf')) + \
-            [P('fonts/calibreSymbols.otf')] + \
-            glob.glob(os.path.join(config_dir, 'fonts', '*.?tf')):
+    load_builtin_fonts.done = True
+    for ff in calibre_font_files():
         if ff.rpartition('.')[-1].lower() in {'ttf', 'otf'}:
             with open(ff, 'rb') as s:
                 # Windows requires font files to be executable for them to be
@@ -763,9 +786,9 @@ def load_builtin_fonts():
                 fid = QFontDatabase.addApplicationFontFromData(s.read())
                 if fid > -1:
                     fam = QFontDatabase.applicationFontFamilies(fid)
-                    fam = set(map(unicode, fam))
-                    if u'calibre Symbols' in fam:
-                        _rating_font = u'calibre Symbols'
+                    fam = set(map(unicode_type, fam))
+                    if 'calibre Symbols' in fam:
+                        _rating_font = 'calibre Symbols'
 
 
 def setup_gui_option_parser(parser):
@@ -806,11 +829,46 @@ def setup_hidpi():
         prints('Not controlling automatic hidpi scaling')
 
 
+def setup_unix_signals(self):
+    if hasattr(os, 'pipe2'):
+        read_fd, write_fd = os.pipe2(os.O_CLOEXEC | os.O_NONBLOCK)
+    else:
+        import fcntl
+        read_fd, write_fd = os.pipe()
+        cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
+        for fd in (read_fd, write_fd):
+            flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+            if flags != -1:
+                fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag)
+            flags = fcntl.fcntl(fd, fcntl.F_GETFL)
+            if flags != -1:
+                fcntl.fcntl(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
+
+    original_handlers = {}
+    for sig in (signal.SIGINT, signal.SIGTERM):
+        original_handlers[sig] = signal.signal(sig, lambda x, y: None)
+        signal.siginterrupt(sig, False)
+    signal.set_wakeup_fd(write_fd)
+    self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
+    self.signal_notifier.setEnabled(True)
+    self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+    return original_handlers
+
+
 class Application(QApplication):
 
     shutdown_signal_received = pyqtSignal()
+    palette_changed = pyqtSignal()
 
-    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs):
+    def __init__(self, args, force_calibre_style=False, override_program_name=None, headless=False, color_prefs=gprefs, windows_app_uid=None):
+        self.ignore_palette_changes = False
+        QNetworkProxyFactory.setUseSystemConfiguration(True)
+        if iswindows:
+            self.windows_app_uid = None
+            if windows_app_uid:
+                windows_app_uid = unicode_type(windows_app_uid)
+                if set_app_uid(windows_app_uid):
+                    self.windows_app_uid = windows_app_uid
         self.file_event_hook = None
         if isfrozen and QT_VERSION <= 0x050700 and 'wayland' in os.environ.get('QT_QPA_PLATFORM', ''):
             os.environ['QT_QPA_PLATFORM'] = 'xcb'
@@ -819,11 +877,13 @@ class Application(QApplication):
         if headless:
             if not args:
                 args = sys.argv[:1]
-            args.extend(['-platformpluginpath', sys.extensions_location, '-platform', 'headless'])
+            args.extend(['-platformpluginpath', plugins_loc, '-platform', 'headless'])
         self.headless = headless
-        qargs = [i.encode('utf-8') if isinstance(i, unicode) else i for i in args]
-        self.pi = plugins['progress_indicator'][0]
-        if not isosx and not headless:
+        qargs = [i.encode('utf-8') if isinstance(i, unicode_type) else i for i in args]
+        self.pi, pi_err = plugins['progress_indicator']
+        if pi_err:
+            raise RuntimeError('Failed to load the progress_indicator C extension, with error: {}'.format(pi_err))
+        if not ismacos and not headless:
             # On OS X high dpi scaling is turned on automatically by the OS, so we dont need to set it explicitly
             setup_hidpi()
         QApplication.setOrganizationName('calibre-ebook.com')
@@ -832,7 +892,13 @@ class Application(QApplication):
         QApplication.setApplicationName(APP_UID)
         if override_program_name and hasattr(QApplication, 'setDesktopFileName'):
             QApplication.setDesktopFileName(override_program_name)
+        QApplication.setAttribute(Qt.AA_ShareOpenGLContexts, True)  # needed for webengine
         QApplication.__init__(self, qargs)
+        sh = self.styleHints()
+        if hasattr(sh, 'setShowShortcutsInContextMenus'):
+            sh.setShowShortcutsInContextMenus(True)
+        if ismacos:
+            plugins['cocoa'][0].disable_cocoa_ui_elements()
         self.setAttribute(Qt.AA_UseHighDpiPixmaps)
         self.setAttribute(Qt.AA_SynthesizeTouchForUnhandledMouseEvents, False)
         try:
@@ -863,7 +929,7 @@ class Application(QApplication):
             if s is not None:
                 font.setStretch(s)
             QApplication.setFont(font)
-        if not isosx and not iswindows:
+        if not ismacos and not iswindows:
             # Qt 5.10.1 on Linux resets the global font on first event loop tick.
             # So workaround it by setting the font once again in a timer.
             font_from_prefs = self.font()
@@ -871,7 +937,7 @@ class Application(QApplication):
         self.line_height = max(12, QFontMetrics(self.font()).lineSpacing())
 
         dl = QLocale(get_lang())
-        if unicode(dl.bcp47Name()) != u'C':
+        if unicode_type(dl.bcp47Name()) != 'C':
             QLocale.setDefault(dl)
         global gui_thread, qt_app
         gui_thread = QThread.currentThread()
@@ -881,7 +947,7 @@ class Application(QApplication):
         self._file_open_paths = []
         self._file_open_lock = RLock()
 
-        if not isosx:
+        if not ismacos:
             # OS X uses a native color dialog that does not support custom
             # colors
             self.color_prefs = color_prefs
@@ -893,7 +959,7 @@ class Application(QApplication):
                 'calibre versions newer than 2.0 do not run on Windows XP. This is'
                 ' because the graphics toolkit calibre uses (Qt 5) crashes a lot'
                 ' on Windows XP. We suggest you stay with <a href="%s">calibre 1.48</a>'
-                ' which works well on Windows XP.') % 'http://download.calibre-ebook.com/1.48.0/', show=True)
+                ' which works well on Windows XP.') % 'https://download.calibre-ebook.com/1.48.0/', show=True)
             raise SystemExit(1)
 
         if iswindows:
@@ -909,7 +975,7 @@ class Application(QApplication):
             # Qt 5 bug: https://bugreports.qt-project.org/browse/QTBUG-41125
             self.aboutToQuit.connect(self.flush_clipboard)
 
-        if isosx:
+        if ismacos:
             cocoa, err = plugins['cocoa']
             if err:
                 raise RuntimeError('Failed to load cocoa plugin with error: {}'.format(err))
@@ -917,6 +983,22 @@ class Application(QApplication):
             if cft >= 0:
                 self.setCursorFlashTime(int(cft))
 
+    def safe_restore_geometry(self, widget, geom):
+        # See https://bugreports.qt.io/browse/QTBUG-77385
+        if not geom:
+            return
+        restored = widget.restoreGeometry(geom)
+        self.ensure_window_on_screen(widget)
+        return restored
+
+    def ensure_window_on_screen(self, widget):
+        screen_rect = self.desktop().availableGeometry(widget)
+        if not widget.geometry().intersects(screen_rect):
+            w = min(widget.width(), screen_rect.width() - 10)
+            h = min(widget.height(), screen_rect.height() - 10)
+            widget.resize(w, h)
+            widget.move((screen_rect.width() - w) // 2, (screen_rect.height() - h) // 2)
+
     def setup_ui_font(self):
         f = QFont(QApplication.font())
         q = (f.family(), f.pointSize())
@@ -938,7 +1020,7 @@ class Application(QApplication):
         try:
             if self.clipboard().ownsClipboard():
                 import ctypes
-                ctypes.WinDLL(b'ole32.dll').OleFlushClipboard()
+                ctypes.WinDLL('ole32.dll').OleFlushClipboard()
         except Exception:
             import traceback
             traceback.print_exc()
@@ -951,40 +1033,96 @@ class Application(QApplication):
 
         load_builtin_fonts()
 
+    def set_dark_mode_palette(self):
+        from calibre.gui2.palette import dark_palette
+        self.set_palette(dark_palette())
+
     def setup_styles(self, force_calibre_style):
-        if iswindows or isosx:
+        if iswindows or ismacos:
             using_calibre_style = gprefs['ui_style'] != 'system'
         else:
             using_calibre_style = os.environ.get('CALIBRE_USE_SYSTEM_THEME', '0') == '0'
         if force_calibre_style:
             using_calibre_style = True
+        if using_calibre_style:
+            use_dark_palette = False
+            if 'CALIBRE_USE_DARK_PALETTE' in os.environ:
+                if not ismacos:
+                    use_dark_palette = os.environ['CALIBRE_USE_DARK_PALETTE'] != '0'
+            else:
+                if iswindows:
+                    use_dark_palette = windows_is_system_dark_mode_enabled()
+            if use_dark_palette:
+                self.set_dark_mode_palette()
+
         self.using_calibre_style = using_calibre_style
         if DEBUG:
             prints('Using calibre Qt style:', self.using_calibre_style)
         if self.using_calibre_style:
             self.load_calibre_style()
+        self.paletteChanged.connect(self.on_palette_change)
+        self.on_palette_change()
+
+    def fix_combobox_text_color(self):
+        # Workaround for https://bugreports.qt.io/browse/QTBUG-75321
+        # Buttontext is set to black for some reason
+        pal = QPalette(self.palette())
+        pal.setColor(pal.ButtonText, pal.color(pal.WindowText))
+        self.ignore_palette_changes = True
+        self.setPalette(pal, 'QComboBox')
+        self.ignore_palette_changes = False
+
+    def set_palette(self, pal):
+        self.ignore_palette_changes = True
+        self.setPalette(pal)
+        # Needed otherwise Qt does not emit the paletteChanged signal when
+        # appearance is changed. And it has to be after current event
+        # processing finishes as of Qt 5.14 otherwise the palette change is
+        # ignored.
+        QTimer.singleShot(1000, lambda: QApplication.instance().setAttribute(Qt.AA_SetPalette, False))
+        self.ignore_palette_changes = False
+
+    def on_palette_change(self):
+        if self.ignore_palette_changes:
+            return
+        self.is_dark_theme = is_dark_theme()
+        self.setProperty('is_dark_theme', self.is_dark_theme)
+        if ismacos and self.is_dark_theme and self.using_calibre_style:
+            QTimer.singleShot(0, self.fix_combobox_text_color)
+        if self.using_calibre_style:
+            ss = 'QTabBar::tab:selected { font-style: italic }\n\n'
+            if self.is_dark_theme:
+                ss += 'QMenu { border: 1px solid palette(shadow); }'
+            self.setStyleSheet(ss)
+        self.palette_changed.emit()
+
+    def stylesheet_for_line_edit(self, is_error=False):
+        return 'QLineEdit { border: 2px solid %s; border-radius: 3px }' % (
+            '#FF2400' if is_error else '#50c878')
 
     def load_calibre_style(self):
         icon_map = self.__icon_map_memory_ = {}
         pcache = {}
-        for k, v in {
-            'DialogYesButton': u'ok.png',
-            'DialogNoButton': u'window-close.png',
-            'DialogCloseButton': u'window-close.png',
-            'DialogOkButton': u'ok.png',
-            'DialogCancelButton': u'window-close.png',
-            'DialogHelpButton': u'help.png',
-            'DialogOpenButton': u'document_open.png',
-            'DialogSaveButton': u'save.png',
-            'DialogApplyButton': u'ok.png',
-            'DialogDiscardButton': u'trash.png',
-            'MessageBoxInformation': u'dialog_information.png',
-            'MessageBoxWarning': u'dialog_warning.png',
-            'MessageBoxCritical': u'dialog_error.png',
-            'MessageBoxQuestion': u'dialog_question.png',
-            'BrowserReload': u'view-refresh.png',
-            'LineEditClearButton': u'clear_left.png',
-        }.iteritems():
+        for k, v in iteritems({
+            'DialogYesButton': 'ok.png',
+            'DialogNoButton': 'window-close.png',
+            'DialogCloseButton': 'window-close.png',
+            'DialogOkButton': 'ok.png',
+            'DialogCancelButton': 'window-close.png',
+            'DialogHelpButton': 'help.png',
+            'DialogOpenButton': 'document_open.png',
+            'DialogSaveButton': 'save.png',
+            'DialogApplyButton': 'ok.png',
+            'DialogDiscardButton': 'trash.png',
+            'MessageBoxInformation': 'dialog_information.png',
+            'MessageBoxWarning': 'dialog_warning.png',
+            'MessageBoxCritical': 'dialog_error.png',
+            'MessageBoxQuestion': 'dialog_question.png',
+            'BrowserReload': 'view-refresh.png',
+            'LineEditClearButton': 'clear_left.png',
+            'ToolBarHorizontalExtensionButton': 'v-ellipsis.png',
+            'ToolBarVerticalExtensionButton': 'h-ellipsis.png',
+        }):
             if v not in pcache:
                 p = I(v)
                 if isinstance(p, bytes):
@@ -993,7 +1131,12 @@ class Application(QApplication):
                 pcache[v] = p
             v = pcache[v]
             icon_map[getattr(QStyle, 'SP_'+k)] = v
-        self.pi.load_style(icon_map)
+        transient_scroller = 0
+        if ismacos:
+            transient_scroller = plugins['cocoa'][0].transient_scroller()
+        icon_map[QStyle.SP_CustomBase + 1] = I('close-for-light-theme.png')
+        icon_map[QStyle.SP_CustomBase + 2] = I('close-for-dark-theme.png')
+        self.pi.load_style(icon_map, transient_scroller)
 
     def _send_file_open_events(self):
         with self._file_open_lock:
@@ -1009,7 +1152,7 @@ class Application(QApplication):
 
     def event(self, e):
         if callable(self.file_event_hook) and e.type() == QEvent.FileOpen:
-            path = unicode(e.file())
+            path = unicode_type(e.file())
             if os.access(path, os.R_OK):
                 with self._file_open_lock:
                     self._file_open_paths.append(path)
@@ -1018,19 +1161,19 @@ class Application(QApplication):
         else:
             return QApplication.event(self, e)
 
-    @dynamic_property
+    @property
     def current_custom_colors(self):
-        from PyQt5.Qt import QColorDialog, QColor
+        from PyQt5.Qt import QColorDialog
 
-        def fget(self):
-            return [col.getRgb() for col in
-                    (QColorDialog.customColor(i) for i in xrange(QColorDialog.customCount()))]
+        return [col.getRgb() for col in
+                    (QColorDialog.customColor(i) for i in range(QColorDialog.customCount()))]
 
-        def fset(self, colors):
-            num = min(len(colors), QColorDialog.customCount())
-            for i in xrange(num):
-                QColorDialog.setCustomColor(i, QColor(*colors[i]))
-        return property(fget=fget, fset=fset)
+    @current_custom_colors.setter
+    def current_custom_colors(self, colors):
+        from PyQt5.Qt import QColorDialog
+        num = min(len(colors), QColorDialog.customCount())
+        for i in range(num):
+            QColorDialog.setCustomColor(i, QColor(*colors[i]))
 
     def read_custom_colors(self):
         colors = self.color_prefs.get('custom_colors_for_color_dialog', None)
@@ -1050,23 +1193,7 @@ class Application(QApplication):
         self.setQuitOnLastWindowClosed(True)
 
     def setup_unix_signals(self):
-        import fcntl
-        if hasattr(os, 'pipe2'):
-            read_fd, write_fd = os.pipe2(os.O_CLOEXEC | os.O_NONBLOCK)
-        else:
-            read_fd, write_fd = os.pipe()
-            cloexec_flag = getattr(fcntl, 'FD_CLOEXEC', 1)
-            for fd in (read_fd, write_fd):
-                flags = fcntl.fcntl(fd, fcntl.F_GETFD)
-                fcntl.fcntl(fd, fcntl.F_SETFD, flags | cloexec_flag | os.O_NONBLOCK)
-
-        for sig in (signal.SIGINT, signal.SIGTERM):
-            signal.signal(sig, lambda x, y: None)
-            signal.siginterrupt(sig, False)
-        signal.set_wakeup_fd(write_fd)
-        self.signal_notifier = QSocketNotifier(read_fd, QSocketNotifier.Read, self)
-        self.signal_notifier.setEnabled(True)
-        self.signal_notifier.activated.connect(self.signal_received, type=Qt.QueuedConnection)
+        setup_unix_signals(self)
 
     def signal_received(self, read_fd):
         try:
@@ -1085,18 +1212,18 @@ def sanitize_env_vars():
     is needed to prevent library conflicts when launching external utilities.'''
 
     if islinux and isfrozen:
-        env_vars = {'LD_LIBRARY_PATH':'/lib', 'QT_PLUGIN_PATH':'/lib/qt_plugins'}
+        env_vars = {'LD_LIBRARY_PATH':'/lib'}
     elif iswindows:
-        env_vars = {k:None for k in 'QT_PLUGIN_PATH'.split()}
-    elif isosx:
+        env_vars = {}
+    elif ismacos:
         env_vars = {k:None for k in (
-                    'FONTCONFIG_FILE FONTCONFIG_PATH QT_PLUGIN_PATH SSL_CERT_FILE').split()}
+                    'FONTCONFIG_FILE FONTCONFIG_PATH SSL_CERT_FILE').split()}
     else:
         env_vars = {}
 
     originals = {x:os.environ.get(x, '') for x in env_vars}
     changed = {x:False for x in env_vars}
-    for var, suffix in env_vars.iteritems():
+    for var, suffix in iteritems(env_vars):
         paths = [x for x in originals[var].split(os.pathsep) if x]
         npaths = [] if suffix is None else [x for x in paths if x != (sys.frozen_path + suffix)]
         if len(npaths) < len(paths):
@@ -1109,7 +1236,7 @@ def sanitize_env_vars():
     try:
         yield
     finally:
-        for var, orig in originals.iteritems():
+        for var, orig in iteritems(originals):
             if changed[var]:
                 if orig:
                     os.environ[var] = orig
@@ -1124,12 +1251,24 @@ def open_url(qurl):
     # Qt 5 requires QApplication to be constructed before trying to use
     # QDesktopServices::openUrl()
     ensure_app()
-    if isinstance(qurl, basestring):
+    if isinstance(qurl, string_or_bytes):
         qurl = QUrl(qurl)
     with sanitize_env_vars():
         QDesktopServices.openUrl(qurl)
 
 
+def safe_open_url(qurl):
+    if isinstance(qurl, string_or_bytes):
+        qurl = QUrl(qurl)
+    if qurl.scheme() in ('', 'file'):
+        path = qurl.toLocalFile()
+        ext = os.path.splitext(path)[-1].lower()[1:]
+        if ext in ('exe', 'com', 'cmd', 'bat', 'sh', 'psh', 'ps1', 'vbs', 'js', 'wsf', 'vba', 'py', 'rb', 'pl', 'app'):
+            prints('Refusing to open file:', path)
+            return
+    open_url(qurl)
+
+
 def get_current_db():
     '''
     This method will try to return the current database in use by the user as
@@ -1161,9 +1300,13 @@ def ensure_app(headless=True):
     with _ea_lock:
         if _store_app is None and QApplication.instance() is None:
             args = sys.argv[:1]
-            has_headless = isosx or islinux or isbsd
+            has_headless = ismacos or islinux or isbsd
             if headless and has_headless:
-                args += ['-platformpluginpath', sys.extensions_location, '-platform', 'headless']
+                args += ['-platformpluginpath', plugins_loc, '-platform', 'headless']
+                if ismacos:
+                    os.environ['QT_MAC_DISABLE_FOREGROUND_APPLICATION_TRANSFORM'] = '1'
+            if headless and iswindows:
+                QApplication.setAttribute(Qt.AA_UseSoftwareOpenGL, True)
             _store_app = QApplication(args)
             if headless and has_headless:
                 _store_app.headless = True
@@ -1183,6 +1326,12 @@ def ensure_app(headless=True):
                 except:
                     pass
             sys.excepthook = eh
+    return _store_app
+
+
+def destroy_app():
+    global _store_app
+    _store_app = None
 
 
 def app_is_headless():
@@ -1232,7 +1381,7 @@ def elided_text(text, font=None, width=300, pos='middle'):
     from PyQt5.Qt import QFontMetrics, QApplication
     fm = QApplication.fontMetrics() if font is None else (font if isinstance(font, QFontMetrics) else QFontMetrics(font))
     delta = 4
-    ellipsis = u'\u2026'
+    ellipsis = '\u2026'
 
     def remove_middle(x):
         mid = len(x) // 2
@@ -1241,7 +1390,7 @@ def elided_text(text, font=None, width=300, pos='middle'):
     chomp = {'middle':remove_middle, 'left':lambda x:(ellipsis + x[delta:]), 'right':lambda x:(x[:-delta] + ellipsis)}[pos]
     while len(text) > delta and fm.width(text) > width:
         text = chomp(text)
-    return unicode(text)
+    return unicode_type(text)
 
 
 def find_forms(srcdir):
@@ -1260,8 +1409,9 @@ def form_to_compiled_form(form):
 
 
 def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
-    import re, cStringIO
+    import re
     from PyQt5.uic import compileUi
+    from polyglot.io import PolyglotStringIO
     forms = find_forms(srcdir)
     if info is None:
         from calibre import prints
@@ -1285,7 +1435,7 @@ def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
         if force_compile or not os.path.exists(compiled_form) or os.stat(form).st_mtime > os.stat(compiled_form).st_mtime:
             if not summary:
                 info('\tCompiling form', form)
-            buf = cStringIO.StringIO()
+            buf = PolyglotStringIO()
             compileUi(form, buf)
             dat = buf.getvalue()
             dat = dat.replace('import images_rc', '')
@@ -1294,7 +1444,8 @@ def build_forms(srcdir, info=None, summary=False, check_for_migration=False):
             dat = dat.replace('_("MMM yyyy")', '"MMM yyyy"')
             dat = dat.replace('_("d MMM yyyy")', '"d MMM yyyy"')
             dat = pat.sub(sub, dat)
-
+            if not isinstance(dat, bytes):
+                dat = dat.encode('utf-8')
             open(compiled_form, 'wb').write(dat)
             num += 1
     if num:
@@ -1310,75 +1461,49 @@ if is_running_from_develop:
 def event_type_name(ev_or_etype):
     from PyQt5.QtCore import QEvent
     etype = ev_or_etype.type() if isinstance(ev_or_etype, QEvent) else ev_or_etype
-    for name, num in vars(QEvent).iteritems():
+    for name, num in iteritems(vars(QEvent)):
         if num == etype:
             return name
     return 'UnknownEventType'
 
 
-def secure_web_page(qwebpage_or_qwebsettings):
-    from PyQt5.QtWebKit import QWebSettings
-    settings = qwebpage_or_qwebsettings if isinstance(qwebpage_or_qwebsettings, QWebSettings) else qwebpage_or_qwebsettings.settings()
-    settings.setAttribute(QWebSettings.JavaEnabled, False)
-    settings.setAttribute(QWebSettings.PluginsEnabled, False)
-    settings.setAttribute(QWebSettings.JavascriptCanOpenWindows, False)
-    settings.setAttribute(QWebSettings.JavascriptCanAccessClipboard, False)
-    settings.setAttribute(QWebSettings.LocalContentCanAccessFileUrls, False)  # ensure javascript cannot read from local files
-    settings.setAttribute(QWebSettings.NotificationsEnabled, False)
-    settings.setThirdPartyCookiePolicy(QWebSettings.AlwaysBlockThirdPartyCookies)
-    settings.setAttribute(QWebSettings.OfflineStorageDatabaseEnabled, False)
-    settings.setAttribute(QWebSettings.LocalStorageEnabled, False)
-    QWebSettings.setOfflineStorageDefaultQuota(0)
-    QWebSettings.setOfflineStoragePath(None)
-    return settings
-
-
 empty_model = QStringListModel([''])
 empty_index = empty_model.index(0)
 
 
-def get_app_uid():
-    import ctypes
-    from ctypes import wintypes
-    lpBuffer = wintypes.LPWSTR()
-    try:
-        AppUserModelID = ctypes.windll.shell32.GetCurrentProcessExplicitAppUserModelID
-    except Exception:  # Vista has no app uids
-        return
-    AppUserModelID.argtypes = [wintypes.LPWSTR]
-    AppUserModelID.restype = wintypes.HRESULT
-    try:
-        AppUserModelID(ctypes.cast(ctypes.byref(lpBuffer), wintypes.LPWSTR))
-    except Exception:
-        return
-    appid = lpBuffer.value
-    ctypes.windll.ole32.CoTaskMemFree(lpBuffer)
-    return appid
-
-
 def set_app_uid(val):
     import ctypes
     from ctypes import wintypes
+    from ctypes import HRESULT
     try:
         AppUserModelID = ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID
     except Exception:  # Vista has no app uids
         return False
     AppUserModelID.argtypes = [wintypes.LPCWSTR]
-    AppUserModelID.restype = wintypes.HRESULT
+    AppUserModelID.restype = HRESULT
     try:
-        AppUserModelID(unicode(val))
+        AppUserModelID(unicode_type(val))
     except Exception as err:
-        prints(u'Failed to set app uid with error:', as_unicode(err))
+        prints('Failed to set app uid with error:', as_unicode(err))
         return False
     return True
 
 
 def add_to_recent_docs(path):
-    from win32com.shell import shell, shellcon
-    path = unicode(path)
-    app_id = get_app_uid()
-    if app_id is None:
-        shell.SHAddToRecentDocs(shellcon.SHARD_PATHW, path)
-    else:
-        item = shell.SHCreateItemFromParsingName(path, None, shell.IID_IShellItem)
-        shell.SHAddToRecentDocs(shellcon.SHARD_APPIDINFO, (item, app_id))
+    app = QApplication.instance()
+    plugins['winutil'][0].add_to_recent_docs(unicode_type(path), app.windows_app_uid)
+
+
+def windows_is_system_dark_mode_enabled():
+    s = QSettings(r"HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Themes\Personalize", QSettings.NativeFormat)
+    if s.status() == QSettings.NoError:
+        return s.value("AppsUseLightTheme") == 0
+    return False
+
+
+def make_view_use_window_background(view):
+    p = view.palette()
+    p.setColor(p.Base, p.color(p.Window))
+    p.setColor(p.AlternateBase, p.color(p.Window))
+    view.setPalette(p)
+    return view
