@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,17 +8,17 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 import shutil, os, weakref, traceback, tempfile, time
 from threading import Thread
 from collections import OrderedDict
-from Queue import Empty
 from io import BytesIO
-from future_builtins import map
+from polyglot.builtins import iteritems, map, unicode_type, string_or_bytes
 
 from PyQt5.Qt import QObject, Qt, pyqtSignal
 
 from calibre import prints, as_unicode
-from calibre.constants import DEBUG, iswindows, isosx, filesystem_encoding
+from calibre.constants import DEBUG, iswindows, ismacos, filesystem_encoding
 from calibre.customize.ui import run_plugins_on_postimport, run_plugins_on_postadd
 from calibre.db.adding import find_books_in_directory, compile_rule
 from calibre.db.utils import find_identical_books
+from calibre.ebooks.metadata import authors_to_sort_string
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
 from calibre.gui2 import error_dialog, warning_dialog, gprefs
@@ -29,10 +28,11 @@ from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs
 from calibre.utils.ipc.pool import Pool, Failure
+from polyglot.queue import Empty
 
 
 def validate_source(source, parent=None):  # {{{
-    if isinstance(source, basestring):
+    if isinstance(source, string_or_bytes):
         if not os.path.exists(source):
             error_dialog(parent, _('Cannot add books'), _(
                 'The path %s does not exist') % source, show=True)
@@ -64,6 +64,10 @@ class Adder(QObject):
         if not validate_source(source, parent):
             return
         QObject.__init__(self, parent)
+        self.author_map_rules = None
+        if gprefs.get('author_map_on_add_rules'):
+            from calibre.ebooks.metadata.author_mapper import compile_rules as acr
+            self.author_map_rules = acr(gprefs['author_map_on_add_rules'])
         self.single_book_per_directory = single_book_per_directory
         self.ignore_opf = False
         self.list_of_archives = list_of_archives
@@ -84,6 +88,7 @@ class Adder(QObject):
         self.report = []
         self.items = []
         self.added_book_ids = set()
+        self.merged_formats_added_to = set()
         self.merged_books = set()
         self.added_duplicate_info = set()
         self.pd.show()
@@ -129,7 +134,7 @@ class Adder(QObject):
             import traceback
             traceback.print_exc()
 
-        if iswindows or isosx:
+        if iswindows or ismacos:
             def find_files(root):
                 for dirpath, dirnames, filenames in os.walk(root):
                     for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
@@ -138,7 +143,7 @@ class Adder(QObject):
                         self.file_groups[len(self.file_groups)] = files
         else:
             def find_files(root):
-                if isinstance(root, type(u'')):
+                if isinstance(root, unicode_type):
                     root = root.encode(filesystem_encoding)
                 for dirpath, dirnames, filenames in os.walk(root):
                     try:
@@ -168,7 +173,7 @@ class Adder(QObject):
             return tdir
 
         try:
-            if isinstance(self.source, basestring):
+            if isinstance(self.source, string_or_bytes):
                 find_files(self.source)
                 self.ignore_opf = True
             else:
@@ -214,7 +219,10 @@ class Adder(QObject):
             self.break_cycles()
             return
         self.pd.max = len(self.file_groups)
-        self.pd.title = _('Reading metadata and adding to library (%d books)...') % self.pd.max
+        self.pd.title = ngettext(
+            'Reading metadata and adding to library (one book)...',
+            'Reading metadata and adding to library ({} books)...',
+            self.pd.max).format(self.pd.max)
         self.pd.msg = ''
         self.pd.value = 0
         self.pool = Pool(name='AddBooks') if self.pool is None else self.pool
@@ -263,7 +271,7 @@ class Adder(QObject):
             except Failure as err:
                 error_dialog(self.pd, _('Cannot add books'), _(
                 'Failed to add some books, click "Show details" for more information.'),
-                det_msg=unicode(err.failure_message) + '\n' + unicode(err.details), show=True)
+                det_msg=unicode_type(err.failure_message) + '\n' + unicode_type(err.details), show=True)
                 self.pd.canceled = True
             else:
                 # All tasks completed
@@ -328,6 +336,15 @@ class Adder(QObject):
         if gprefs.get('tag_map_on_add_rules'):
             from calibre.ebooks.metadata.tag_mapper import map_tags
             mi.tags = map_tags(mi.tags, gprefs['tag_map_on_add_rules'])
+        if self.author_map_rules:
+            from calibre.ebooks.metadata.author_mapper import map_authors
+            new_authors = map_authors(mi.authors, self.author_map_rules)
+            if new_authors != mi.authors:
+                mi.authors = new_authors
+                if self.db is None:
+                    mi.author_sort = authors_to_sort_string(mi.authors)
+                else:
+                    mi.author_sort = self.db.author_sort_from_authors(mi.authors)
 
         self.pd.msg = mi.title
 
@@ -366,6 +383,7 @@ class Adder(QObject):
             ib_fmts = {fmt.upper() for fmt in self.db.formats(identical_book_id)}
             seen_fmts |= ib_fmts
             self.add_formats(identical_book_id, paths, mi, replace=replace)
+            self.merged_formats_added_to.add(identical_book_id)
         if gprefs['automerge'] == 'new record':
             incoming_fmts = {path.rpartition(os.extsep)[-1].upper() for path in paths}
             if incoming_fmts.intersection(seen_fmts):
@@ -418,7 +436,7 @@ class Adder(QObject):
     def add_formats(self, book_id, paths, mi, replace=True, is_an_add=False):
         fmap = {p.rpartition(os.path.extsep)[-1].lower():p for p in paths}
         fmt_map = {}
-        for fmt, path in fmap.iteritems():
+        for fmt, path in iteritems(fmap):
             # The onimport plugins have already been run by the read metadata
             # worker
             if self.ignore_opf and fmt.lower() == 'opf':
@@ -471,9 +489,9 @@ class Adder(QObject):
                 'Failed to add any books, click "Show details" for more information')
             d(self.pd, _('Errors while adding'), msg, det_msg='\n'.join(self.report), show=True)
 
-        if gprefs['manual_add_auto_convert'] and self.added_book_ids and self.parent() is not None:
-            self.parent().iactions['Convert Books'].auto_convert_auto_add(
-                self.added_book_ids)
+        potentially_convertible = self.added_book_ids | self.merged_formats_added_to
+        if gprefs['manual_add_auto_convert'] and potentially_convertible and self.parent() is not None:
+            self.parent().iactions['Convert Books'].auto_convert_auto_add(potentially_convertible)
 
         try:
             if callable(self.callback):
