@@ -1,14 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
-from future_builtins import map
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import traceback, cPickle, copy, os
+import traceback, copy, os
 from collections import OrderedDict
 
 from PyQt5.Qt import (QAbstractItemModel, QIcon, QFont, Qt,
@@ -18,11 +16,14 @@ from calibre.constants import config_dir
 from calibre.ebooks.metadata import rating_to_stars
 from calibre.gui2 import gprefs, config, error_dialog, file_icon_provider
 from calibre.db.categories import Tag
-from calibre.utils.config import tweaks
-from calibre.utils.icu import sort_key, lower, strcmp, collation_order
+from calibre.utils.config import tweaks, prefs
+from calibre.utils.icu import sort_key, lower, strcmp, collation_order, primary_strcmp, primary_contains, contains
 from calibre.library.field_metadata import category_icon_map
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.utils.formatter import EvalFormatter
+from calibre.utils.serialize import json_dumps, json_loads
+from polyglot.builtins import iteritems, itervalues, map, range, unicode_type
+
 
 TAG_SEARCH_STATES = {'clear': 0, 'mark_plus': 1, 'mark_plusplus': 2,
                      'mark_minus': 3, 'mark_minusminus': 4}
@@ -58,6 +59,8 @@ class TagTreeItem(object):  # {{{
         self.blank = QIcon()
         self.is_gst = False
         self.boxed = False
+        self.temporary = False
+        self.can_be_edited = False
         self.icon_state_map = list(icon_map)
         if self.parent is not None:
             self.parent.append(self)
@@ -112,9 +115,9 @@ class TagTreeItem(object):  # {{{
         if self.type == self.ROOT:
             return 'ROOT'
         if self.type == self.CATEGORY:
-            return 'CATEGORY(category_key={!r}, name={!r}, num_children={!r})'.format(
-                self.category_key, self.name, len(self.children))
-        return 'TAG(name=%r)'%self.tag.name
+            return 'CATEGORY(category_key={!r}, name={!r}, num_children={!r}, temp={!r})'.format(
+                self.category_key, self.name, len(self.children), self.temporary)
+        return 'TAG(name={!r}), temp={!r})'.format(self.tag.name, self.temporary)
 
     def row(self):
         if self.parent is not None:
@@ -173,7 +176,7 @@ class TagTreeItem(object):  # {{{
         if role == Qt.FontRole:
             return bf()
         if role == Qt.ToolTipRole:
-            return self.tooltip
+            return self.tooltip if gprefs['tag_browser_show_tooltips'] else None
         if role == DRAG_IMAGE_ROLE:
             self.ensure_icon()
             return self.icon_state_map[0]
@@ -191,7 +194,7 @@ class TagTreeItem(object):  # {{{
             else:
                 name = tag.name
         if role == Qt.DisplayRole:
-            return unicode(name)
+            return unicode_type(name)
         if role == Qt.EditRole:
             return (tag.original_name)
         if role == Qt.DecorationRole:
@@ -199,21 +202,23 @@ class TagTreeItem(object):  # {{{
                 self.ensure_icon()
             return self.icon_state_map[tag.state]
         if role == Qt.ToolTipRole:
-            tt = [self.tooltip] if self.tooltip else []
-            if tag.original_categories:
-                tt.append('%s:%s' % (','.join(tag.original_categories), tag.original_name))
-            else:
-                tt.append('%s:%s' % (tag.category, tag.original_name))
-            ar = self.average_rating
-            if ar:
-                tt.append(_('Average rating for books in this category: %.1f') % ar)
-            elif self.type == self.TAG and ar is not None:
-                tt.append(_('Books in this category are unrated'))
-            if self.type == self.TAG and self.tag.category == 'search':
-                tt.append(_('Search expression:') + ' ' + self.tag.search_expression)
-            if self.type == self.TAG:
-                tt.append(_('Number of books: %s') % self.item_count)
-            return '\n'.join(tt)
+            if gprefs['tag_browser_show_tooltips']:
+                tt = [self.tooltip] if self.tooltip else []
+                if tag.original_categories:
+                    tt.append('%s:%s' % (','.join(tag.original_categories), tag.original_name))
+                else:
+                    tt.append('%s:%s' % (tag.category, tag.original_name))
+                ar = self.average_rating
+                if ar:
+                    tt.append(_('Average rating for books in this category: %.1f') % ar)
+                elif self.type == self.TAG and ar is not None:
+                    tt.append(_('Books in this category are unrated'))
+                if self.type == self.TAG and self.tag.category == 'search':
+                    tt.append(_('Search expression:') + ' ' + self.tag.search_expression)
+                if self.type == self.TAG and self.tag.category != 'search':
+                    tt.append(_('Number of books: %s') % self.item_count)
+                return '\n'.join(tt)
+            return None
         if role == DRAG_IMAGE_ROLE:
             self.ensure_icon()
             return self.icon_state_map[0]
@@ -302,7 +307,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         self.node_map = {}
         self.category_nodes = []
         self.category_custom_icons = {}
-        for k, v in self.prefs['tags_browser_category_icons'].iteritems():
+        for k, v in iteritems(self.prefs['tags_browser_category_icons']):
             icon = QIcon(os.path.join(config_dir, 'tb_icons', v))
             if len(icon.availableSizes()) > 0:
                 self.category_custom_icons[k] = icon
@@ -370,7 +375,7 @@ class TagsModel(QAbstractItemModel):  # {{{
 
     def rebuild_node_tree(self, state_map={}):
         if self._build_in_progress:
-            print ('Tag browser build already in progress')
+            print('Tag browser build already in progress')
             traceback.print_stack()
             return
         # traceback.print_stack()
@@ -382,11 +387,12 @@ class TagsModel(QAbstractItemModel):  # {{{
         self._build_in_progress = False
 
     def _run_rebuild(self, state_map={}):
-        for node in self.node_map.itervalues():
+        for node in itervalues(self.node_map):
             node.break_cycles()
         del node  # Clear reference to node in the current frame
         self.node_map.clear()
         self.category_nodes = []
+        self.hierarchical_categories = {}
         self.root_item = self.create_node(icon_map=self.icon_state_map)
         self._rebuild_node_tree(state_map=state_map)
 
@@ -394,7 +400,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         # Note that _get_category_nodes can indirectly change the
         # user_categories dict.
         data = self._get_category_nodes(config['sort_tags_by'])
-        gst = self.db.prefs.get('grouped_search_terms', {})
+        gst = self.db.new_api.pref('grouped_search_terms', {})
 
         last_category_node = None
         category_node_map = {}
@@ -421,8 +427,8 @@ class TagsModel(QAbstractItemModel):  # {{{
 
             if self.category_custom_icons.get(key, None) is None:
                 self.category_custom_icons[key] = QIcon(I(
-                    category_icon_map['gst'] if is_gst else
-                    category_icon_map.get(key, category_icon_map['custom:'])))
+                    category_icon_map['gst'] if is_gst else category_icon_map.get(
+                        key, (category_icon_map['user:'] if key.startswith('@') else category_icon_map['custom:']))))
 
             if key.startswith('@'):
                 path_parts = [p for p in key.split('.')]
@@ -470,7 +476,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         intermediate_nodes = {}
 
         if data is None:
-            print ('_create_node_tree: no data!')
+            print('_create_node_tree: no data!')
             traceback.print_stack()
             return
 
@@ -523,10 +529,9 @@ class TagsModel(QAbstractItemModel):  # {{{
                 last_ordnum = 0
                 last_c = ' '
                 for idx,tag in enumerate(data[key]):
-                    if not tag.sort:
-                        c = ' '
-                    else:
-                        c = icu_upper(tag.sort)
+                    # Deal with items that don't have sorts, such as formats
+                    t = tag.sort if tag.sort else tag.name
+                    c = icu_upper(t) if t else ' '
                     ordnum, ordlen = collation_order(c)
                     if last_ordnum != ordnum:
                         last_c = c[0:ordlen]
@@ -535,10 +540,7 @@ class TagsModel(QAbstractItemModel):  # {{{
             top_level_component = 'z' + data[key][0].original_name
 
             last_idx = -collapse
-            category_is_hierarchical = not (
-                key in ['authors', 'publisher', 'news', 'formats', 'rating'] or
-                key not in self.db.prefs.get('categories_using_hierarchy', []) or
-                config['sort_tags_by'] != 'name')
+            category_is_hierarchical = self.is_key_a_hierarchical_category(key)
 
             for idx,tag in enumerate(data[key]):
                 components = None
@@ -570,7 +572,13 @@ class TagsModel(QAbstractItemModel):  # {{{
                                 d['first'] = ct2
                             else:
                                 d = {'first': tag}
+                                # Some nodes like formats and identifiers don't
+                                # have sort set. Fix that so the template will work
+                                if d['first'].sort is None:
+                                    d['first'].sort = tag.name
                                 d['last'] = data[key][last]
+                                if d['last'].sort is None:
+                                    d['last'].sort = data[key][last].name
 
                             name = eval_formatter.safe_format(collapse_template,
                                                         d, '##TAG_VIEW##', None)
@@ -713,6 +721,22 @@ class TagsModel(QAbstractItemModel):  # {{{
             p = p.parent
         return p.tag.category.startswith('@')
 
+    def is_key_a_hierarchical_category(self, key):
+        result = self.hierarchical_categories.get(key)
+        if result is None:
+            result = not (
+                    key in ['authors', 'publisher', 'news', 'formats', 'rating'] or
+                    key not in self.db.new_api.pref('categories_using_hierarchy', []) or
+                    config['sort_tags_by'] != 'name')
+            self.hierarchical_categories[key] = result
+        return result
+
+    def is_index_on_a_hierarchical_category(self, index):
+        if not index.isValid():
+            return False
+        p = self.get_node(index)
+        return self.is_key_a_hierarchical_category(p.tag.category)
+
     # Drag'n Drop {{{
     def mimeTypes(self):
         return ["application/calibre+from_library",
@@ -737,13 +761,13 @@ class TagsModel(QAbstractItemModel):  # {{{
                 data.append(d)
             else:
                 data.append(None)
-        raw = bytearray(cPickle.dumps(data, -1))
+        raw = bytearray(json_dumps(data))
         ans = QMimeData()
         ans.setData('application/calibre+from_tag_browser', raw)
         return ans
 
     def dropMimeData(self, md, action, row, column, parent):
-        fmts = set([unicode(x) for x in md.formats()])
+        fmts = {unicode_type(x) for x in md.formats()}
         if not fmts.intersection(set(self.mimeTypes())):
             return False
         if "application/calibre+from_library" in fmts:
@@ -757,15 +781,46 @@ class TagsModel(QAbstractItemModel):  # {{{
         if not parent.isValid():
             return False
         dest = self.get_node(parent)
-        if dest.type != TagTreeItem.CATEGORY:
-            return False
         if not md.hasFormat('application/calibre+from_tag_browser'):
             return False
-        data = str(md.data('application/calibre+from_tag_browser'))
-        src = cPickle.loads(data)
-        for s in src:
+        data = bytes(md.data('application/calibre+from_tag_browser'))
+        src = json_loads(data)
+        if len(src) == 1:
+            # Check to see if this is a hierarchical rename
+            s = src[0]
+            # This check works for both hierarchical and user categories.
+            # We can drag only tag items.
             if s[0] != TagTreeItem.TAG:
                 return False
+            src_index = self.index_for_path(s[5])
+            if src_index == parent:
+                # dropped on itself
+                return False
+            src_item = self.get_node(src_index)
+            dest_item = parent.data(Qt.UserRole)
+            # Here we do the real work. If src is a tag, src == dest, and src
+            # is hierarchical then we can do a rename.
+            if (src_item.type == TagTreeItem.TAG and
+                    src_item.tag.category == dest_item.tag.category and
+                    self.is_key_a_hierarchical_category(src_item.tag.category)):
+                key = s[1]
+                # work out the part of the source name to use in the rename
+                # It isn't necessarily a simple name but might be the remaining
+                # levels of the hierarchy
+                part = src_item.tag.original_name.rpartition('.')
+                src_simple_name = part[2]
+                # work out the new prefix, the destination node name
+                if dest.type == TagTreeItem.TAG:
+                    new_name = dest_item.tag.original_name + '.' + src_simple_name
+                else:
+                    new_name = src_simple_name
+                # In d&d renames always use the vl. This might be controversial.
+                src_item.use_vl = True
+                self.rename_item(src_item, key, new_name)
+                return True
+        # Should be working with a user category
+        if dest.type != TagTreeItem.CATEGORY:
+            return False
         return self.move_or_copy_item_to_user_category(src, dest, action)
 
     def move_or_copy_item_to_user_category(self, src, dest, action):
@@ -812,7 +867,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                                              is_uc, dest_key, c)
             return copied
 
-        user_cats = self.db.prefs.get('user_categories', {})
+        user_cats = self.db.new_api.pref('user_categories', {})
         path = None
         for s in src:
             src_parent, src_parent_is_gst = s[1:3]
@@ -854,7 +909,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                                                    fm['datatype'] == 'composite' and
                                                    fm['display'].get('make_category', False)))):
                     mime = 'application/calibre+from_library'
-                    ids = list(map(int, str(md.data(mime)).split()))
+                    ids = list(map(int, md.data(mime).data().split()))
                     self.handle_drop(node, ids)
                     return True
             elif node.type == TagTreeItem.CATEGORY:
@@ -868,17 +923,17 @@ class TagsModel(QAbstractItemModel):  # {{{
                              (fm_src['datatype'] == 'composite' and
                               fm_src['display'].get('make_category', False))):
                         mime = 'application/calibre+from_library'
-                        ids = list(map(int, str(md.data(mime)).split()))
+                        ids = list(map(int, md.data(mime).data().split()))
                         self.handle_user_category_drop(node, ids, md.column_name)
                         return True
         return False
 
     def handle_user_category_drop(self, on_node, ids, column):
-        categories = self.db.prefs.get('user_categories', {})
+        categories = self.db.new_api.pref('user_categories', {})
         cat_contents = categories.get(on_node.category_key[1:], None)
         if cat_contents is None:
             return
-        cat_contents = set([(v, c) for v,c,ign in cat_contents])
+        cat_contents = {(v, c) for v,c,ign in cat_contents}
 
         fm_src = self.db.metadata_for_field(column)
         label = fm_src['label']
@@ -901,7 +956,7 @@ class TagsModel(QAbstractItemModel):  # {{{
             if value:
                 if not isinstance(value, list):
                     value = [value]
-                cat_contents |= set([(v, column) for v in value])
+                cat_contents |= {(v, column) for v in value}
 
         categories[on_node.category_key[1:]] = [[v, c, 0] for v,c in cat_contents]
         self.db.new_api.set_pref('user_categories', categories)
@@ -991,9 +1046,19 @@ class TagsModel(QAbstractItemModel):  # {{{
             self.restriction_error.emit()
 
         if self.filter_categories_by:
+            if self.filter_categories_by.startswith('='):
+                use_exact_match = True
+                filter_by = self.filter_categories_by[1:]
+            else:
+                use_exact_match = False
+                filter_by = self.filter_categories_by
             for category in data.keys():
-                data[category] = [t for t in data[category]
-                        if lower(t.name).find(self.filter_categories_by) >= 0]
+                if use_exact_match:
+                    data[category] = [t for t in data[category]
+                        if lower(t.name) == filter_by]
+                else:
+                    data[category] = [t for t in data[category]
+                        if lower(t.name).find(filter_by) >= 0]
 
         # Build a dict of the keys that have data
         tb_categories = self.db.field_metadata
@@ -1007,7 +1072,7 @@ class TagsModel(QAbstractItemModel):  # {{{
             if not isinstance(order, dict):
                 raise TypeError()
         except:
-            print ('Tweak tag_browser_category_order is not valid. Ignored')
+            print('Tweak tag_browser_category_order is not valid. Ignored')
             order = {'*': 100}
         defvalue = order.get('*', 100)
         self.row_map = sorted(self.categories, key=lambda x: order.get(x, defvalue))
@@ -1027,7 +1092,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         Here to trap usages of refresh in the old architecture. Can eventually
         be removed.
         '''
-        print ('TagsModel: refresh called!')
+        print('TagsModel: refresh called!')
         traceback.print_stack()
         return False
 
@@ -1046,6 +1111,9 @@ class TagsModel(QAbstractItemModel):  # {{{
                 id(internal_pointer))
         return idx
 
+    def category_row_map(self):
+        return {category.category_key:row for row, category in enumerate(self.root_item.children)}
+
     def index_for_category(self, name):
         for row, category in enumerate(self.root_item.children):
             if category.category_key == name:
@@ -1066,7 +1134,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         # set up to reposition at the same item. We can do this except if
         # working with the last item and that item is deleted, in which case
         # we position at the parent label
-        val = unicode(value or '').strip()
+        val = unicode_type(value or '').strip()
         if not val:
             error_dialog(self.gui_parent, _('Item is blank'),
                         _('An item cannot be set to nothing. Delete it instead.')).exec_()
@@ -1079,7 +1147,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                       'renaming User categories'), show=True)
                 return False
 
-            user_cats = self.db.prefs.get('user_categories', {})
+            user_cats = self.db.new_api.pref('user_categories', {})
             user_cat_keys_lower = [icu_lower(k) for k in user_cats]
             ckey = item.category_key[1:]
             ckey_lower = icu_lower(ckey)
@@ -1094,7 +1162,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                 self.use_position_based_index_on_next_recount = True
                 return True
 
-            for c in sorted(user_cats.keys(), key=sort_key):
+            for c in sorted(list(user_cats.keys()), key=sort_key):
                 if icu_lower(c).startswith(ckey_lower):
                     if len(c) == len(ckey):
                         if strcmp(ckey, nkey) != 0 and \
@@ -1118,7 +1186,6 @@ class TagsModel(QAbstractItemModel):  # {{{
             return True
 
         key = item.tag.category
-        name = item.tag.original_name
         # make certain we know about the item's category
         if key not in self.db.field_metadata:
             return False
@@ -1133,30 +1200,57 @@ class TagsModel(QAbstractItemModel):  # {{{
                     _('The saved search name %s is already used.')%val).exec_()
                 return False
             self.use_position_based_index_on_next_recount = True
-            self.db.saved_search_rename(unicode(item.data(role) or ''), val)
+            self.db.saved_search_rename(unicode_type(item.data(role) or ''), val)
             item.tag.name = val
             self.search_item_renamed.emit()  # Does a refresh
         else:
-            self.use_position_based_index_on_next_recount = True
-            restrict_to_book_ids=self.get_book_ids_to_use() if item.use_vl else None
-            self.db.new_api.rename_items(key, {item.tag.id: val},
-                                         restrict_to_book_ids=restrict_to_book_ids)
-            self.tag_item_renamed.emit()
-            item.tag.name = val
-            item.tag.state = TAG_SEARCH_STATES['clear']
-            self.use_position_based_index_on_next_recount = True
-            if not restrict_to_book_ids:
-                self.rename_item_in_all_user_categories(name, key, val)
-            self.refresh_required.emit()
+            self.rename_item(item, key, val)
         return True
 
+    def rename_item(self, item, key, to_what):
+        def do_one_item(lookup_key, an_item, original_name, new_name, restrict_to_books):
+            self.use_position_based_index_on_next_recount = True
+            self.db.new_api.rename_items(lookup_key, {an_item.tag.id: new_name},
+                                         restrict_to_book_ids=restrict_to_books)
+            self.tag_item_renamed.emit()
+            an_item.tag.name = new_name
+            an_item.tag.state = TAG_SEARCH_STATES['clear']
+            self.use_position_based_index_on_next_recount = True
+            if not restrict_to_books:
+                self.rename_item_in_all_user_categories(original_name,
+                                                        lookup_key, new_name)
+
+        children = item.all_children()
+        restrict_to_book_ids=self.get_book_ids_to_use() if item.use_vl else None
+        if item.tag.is_editable and len(children) == 0:
+            # Leaf node, just do it.
+            do_one_item(key, item, item.tag.original_name, to_what, restrict_to_book_ids)
+        else:
+            # Middle node of a hierarchy
+            search_name = item.tag.original_name
+            # Clear any search icons on the original tag
+            if item.parent.type == TagTreeItem.TAG:
+                item.parent.tag.state = TAG_SEARCH_STATES['clear']
+            # It might also be a leaf
+            if item.tag.is_editable:
+                do_one_item(key, item, item.tag.original_name, to_what, restrict_to_book_ids)
+            # Now do the children
+            for child_item in children:
+                from calibre.utils.icu import startswith
+                if (child_item.tag.is_editable and
+                        startswith(child_item.tag.original_name, search_name)):
+                    new_name = to_what + child_item.tag.original_name[len(search_name):]
+                    do_one_item(key, child_item, child_item.tag.original_name,
+                                new_name, restrict_to_book_ids)
+        self.refresh_required.emit()
+
     def rename_item_in_all_user_categories(self, item_name, item_category, new_name):
         '''
         Search all User categories for items named item_name with category
         item_category and rename them to new_name. The caller must arrange to
         redisplay the tree as appropriate.
         '''
-        user_cats = self.db.prefs.get('user_categories', {})
+        user_cats = self.db.new_api.pref('user_categories', {})
         for k in user_cats.keys():
             new_contents = []
             for tup in user_cats[k]:
@@ -1173,7 +1267,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         item_category and delete them. The caller must arrange to redisplay the
         tree as appropriate.
         '''
-        user_cats = self.db.prefs.get('user_categories', {})
+        user_cats = self.db.new_api.pref('user_categories', {})
         for cat in user_cats.keys():
             self.delete_item_from_user_category(cat, item_name, item_category,
                                                 user_categories=user_cats)
@@ -1184,7 +1278,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         if user_categories is not None:
             user_cats = user_categories
         else:
-            user_cats = self.db.prefs.get('user_categories', {})
+            user_cats = self.db.new_api.pref('user_categories', {})
         new_contents = []
         for tup in user_cats[category]:
             if tup[0] != item_name or tup[1] != item_category:
@@ -1201,7 +1295,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         if index.isValid():
             node = self.data(index, Qt.UserRole)
             if node.type == TagTreeItem.TAG:
-                if node.tag.is_editable:
+                if node.tag.is_editable or node.tag.is_hierarchical:
                     ans |= Qt.ItemIsDragEnabled
                 fm = self.db.metadata_for_field(node.tag.category)
                 if node.tag.category in \
@@ -1455,6 +1549,12 @@ class TagsModel(QAbstractItemModel):  # {{{
         self.path_found = None
         if start_path is None:
             start_path = []
+        if prefs['use_primary_find_in_search']:
+            final_strcmp = primary_strcmp
+            final_contains = primary_contains
+        else:
+            final_strcmp = strcmp
+            final_contains = contains
 
         def process_tag(depth, tag_index, tag_item, start_path):
             path = self.path_for_index(tag_index)
@@ -1464,8 +1564,8 @@ class TagsModel(QAbstractItemModel):  # {{{
             if tag is None:
                 return False
             name = tag.original_name
-            if (equals_match and strcmp(name, txt) == 0) or \
-                    (not equals_match and lower(name).find(txt) >= 0):
+            if (equals_match and final_strcmp(name, txt) == 0) or \
+                    (not equals_match and final_contains(txt, name)):
                 self.path_found = path
                 return True
             for i,c in enumerate(tag_item.children):
@@ -1481,7 +1581,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                 if path[depth] > start_path[depth]:
                     start_path = path
             my_key = self.get_node(category_index).category_key
-            for j in xrange(self.rowCount(category_index)):
+            for j in range(self.rowCount(category_index)):
                 tag_index = self.index(j, 0, category_index)
                 tag_item = self.get_node(tag_index)
                 if tag_item.type == TagTreeItem.CATEGORY:
@@ -1492,7 +1592,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                         return True
             return False
 
-        for i in xrange(self.rowCount(QModelIndex())):
+        for i in range(self.rowCount(QModelIndex())):
             if process_level(0, self.index(i, 0, QModelIndex()), start_path):
                 break
         return self.path_found
@@ -1506,7 +1606,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         if not key:
             return None
 
-        for i in xrange(self.rowCount(parent)):
+        for i in range(self.rowCount(parent)):
             idx = self.index(i, 0, parent)
             node = self.get_node(idx)
             if node.type == TagTreeItem.CATEGORY:
@@ -1536,7 +1636,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                 process_tag(self.index(i, 0, tag_index), c)
 
         def process_level(category_index):
-            for j in xrange(self.rowCount(category_index)):
+            for j in range(self.rowCount(category_index)):
                 tag_index = self.index(j, 0, category_index)
                 tag_item = self.get_node(tag_index)
                 if tag_item.boxed:
@@ -1547,7 +1647,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                 else:
                     process_tag(tag_index, tag_item)
 
-        for i in xrange(self.rowCount(QModelIndex())):
+        for i in range(self.rowCount(QModelIndex())):
             process_level(self.index(i, 0, QModelIndex()))
 
     # }}}
