@@ -1,17 +1,17 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import string
-from future_builtins import map
+from polyglot.builtins import iteritems, map
 
 from calibre.utils.config import JSONConfig
 from calibre.spell.dictionary import Dictionaries, parse_lang_code
 
+CONTAINER_DND_MIMETYPE = 'application/x-calibre-container-name-list'
 tprefs = JSONConfig('tweak_book_gui')
 d = tprefs.defaults
 
@@ -31,13 +31,16 @@ d['dock_top_left'] = 'horizontal'
 d['dock_top_right'] = 'horizontal'
 d['dock_bottom_left'] = 'horizontal'
 d['dock_bottom_right'] = 'horizontal'
-d['preview_serif_family'] = 'Liberation Serif'
-d['preview_sans_family'] = 'Liberation Sans'
-d['preview_mono_family'] = 'Liberation Mono'
+d['engine_preview_serif_family'] = None
+d['engine_preview_sans_family'] = None
+d['engine_preview_mono_family'] = None
 d['preview_standard_font_family'] = 'serif'
 d['preview_base_font_size'] = 18
 d['preview_mono_font_size'] = 14
 d['preview_minimum_font_size'] = 8
+d['preview_background'] = 'auto'
+d['preview_foreground'] = 'auto'
+d['preview_link_color'] = 'auto'
 d['remove_existing_links_when_linking_sheets'] = True
 d['charmap_favorites'] = list(map(ord, '\xa0\u2002\u2003\u2009\xad' '‘’“”‹›«»‚„' '—–§¶†‡©®™' '→⇒•·°±−×÷¼½½¾' '…µ¢£€¿¡¨´¸ˆ˜' 'ÀÁÂÃÄÅÆÇÈÉÊË' 'ÌÍÎÏÐÑÒÓÔÕÖØ' 'ŒŠÙÚÛÜÝŸÞßàá' 'âãäåæçèéêëìí' 'îïðñòóôõöøœš' 'ùúûüýÿþªºαΩ∞'))  # noqa
 d['folders_for_types'] = {'style':'styles', 'image':'images', 'font':'fonts', 'audio':'audio', 'video':'video'}
@@ -49,6 +52,7 @@ d['spell_check_case_sensitive_sort'] = False
 d['inline_spell_check'] = True
 d['custom_themes'] = {}
 d['remove_unused_classes'] = False
+d['merge_identical_selectors'] = False
 d['global_book_toolbar'] = [
 'new-file', 'open-book',  'save-book', None, 'global-undo', 'global-redo', 'create-checkpoint', None, 'donate', 'user-manual']
 d['global_tools_toolbar'] = [
@@ -57,7 +61,7 @@ d['global_tools_toolbar'] = [
 ]
 d['global_plugins_toolbar'] = []
 d['editor_common_toolbar'] = [('editor-' + x) if x else None for x in ('undo', 'redo', None, 'cut', 'copy', 'paste', 'smart-comment')]
-d['editor_css_toolbar'] = ['pretty-current', 'insert-image']
+d['editor_css_toolbar'] = ['pretty-current', 'editor-sort-css', 'insert-image']
 d['editor_xml_toolbar'] = ['pretty-current', 'insert-tag']
 d['editor_html_toolbar'] = ['fix-html-current', 'pretty-current', 'insert-image', 'insert-hyperlink', 'insert-tag', 'change-paragraph']
 d['editor_format_toolbar'] = [('format-text-' + x) if x else x for x in (
@@ -74,21 +78,29 @@ d['toolbar_icon_size'] = 24
 d['insert_full_screen_image'] = False
 d['preserve_aspect_ratio_when_inserting_image'] = False
 d['file_list_shows_full_pathname'] = False
+d['auto_link_stylesheets'] = True
+d['check_external_link_anchors'] = True
 del d
 
 ucase_map = {l:string.ascii_uppercase[i] for i, l in enumerate(string.ascii_lowercase)}
+
+
 def capitalize(x):
     return ucase_map[x[0]] + x[1:]
 
+
 _current_container = None
 
+
 def current_container():
     return _current_container
 
+
 def set_current_container(container):
     global _current_container
     _current_container = container
 
+
 class NonReplaceDict(dict):
 
     def __setitem__(self, k, v):
@@ -96,6 +108,7 @@ class NonReplaceDict(dict):
             raise ValueError('The key %s is already present' % k)
         dict.__setitem__(self, k, v)
 
+
 actions = NonReplaceDict()
 editors = NonReplaceDict()
 toolbar_actions = NonReplaceDict()
@@ -105,11 +118,13 @@ editor_toolbar_actions = {
 TOP = object()
 dictionaries = Dictionaries()
 
+
 def editor_name(editor):
-    for n, ed in editors.iteritems():
+    for n, ed in iteritems(editors):
         if ed is editor:
             return n
 
+
 def set_book_locale(lang):
     dictionaries.initialize()
     try:
@@ -121,6 +136,7 @@ def set_book_locale(lang):
     from calibre.gui2.tweak_book.editor.syntax.html import refresh_spell_check_status
     refresh_spell_check_status()
 
+
 def verify_link(url, name=None):
     if _current_container is None or name is None:
         return None
@@ -136,9 +152,10 @@ def verify_link(url, name=None):
         return True
     return False
 
+
 def update_mark_text_action(ed=None):
     has_mark = False
     if ed is not None and ed.has_line_numbers:
         has_mark = bool(ed.selected_text) or not ed.has_marked_text
     ac = actions['mark-selected-text']
-    ac.setText(ac.default_text if has_mark else _('Unmark marked text'))
+    ac.setText(ac.default_text if has_mark else _('&Unmark marked text'))
